@@ -1,7 +1,6 @@
-/* $Id: cpl_image_basic_body.h,v 1.107 2013-10-22 11:50:33 llundin Exp $
- *
+/*
  * This file is part of the ESO Common Pipeline Library
- * Copyright (C) 2001-2008 European Southern Observatory
+ * Copyright (C) 2001-2017 European Southern Observatory
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -229,29 +228,6 @@ static cpl_error_code CPL_TYPE_ADD(cpl_image_exponential)(cpl_image *, double);
 
 static cpl_error_code CPL_TYPE_ADD(cpl_image_logarithm)(cpl_image *, double);
 
-static void CPL_TYPE_ADD(cpl_image_flip)(CPL_TYPE *, size_t) CPL_ATTR_NONNULL;
-
-/*----------------------------------------------------------------------------*/
-/**
-  @internal
-  @brief    Flip (reverse the order of) the elements
-  @param    self  The array
-  @param    n     The (positive) array size
-  @return   void
-
- */
-/*----------------------------------------------------------------------------*/
-static void CPL_TYPE_ADD(cpl_image_flip)(CPL_TYPE * self, size_t n)
-{
-    size_t i;
-
-    for (i = 0; i < n/2; i++) {
-        const CPL_TYPE tmp = self[i];
-        self[i] = self[n-1-i];
-        self[n-1-i] = tmp;
-    }
-}
-
 /*----------------------------------------------------------------------------*/
 /**
   @internal
@@ -673,7 +649,6 @@ CPL_TYPE_ADD(cpl_image_collapse_window_create)(const cpl_image * self,
     size_t             i, j;
 
 
-    cpl_ensure(direction == 0 || direction == 1, CPL_ERROR_ILLEGAL_INPUT, NULL);
     cpl_ensure(llx >= 1,        CPL_ERROR_ILLEGAL_INPUT, NULL);
     cpl_ensure(lly >= 1,        CPL_ERROR_ILLEGAL_INPUT, NULL);
     cpl_ensure(urx <= self->nx, CPL_ERROR_ILLEGAL_INPUT, NULL);
@@ -747,6 +722,9 @@ CPL_TYPE_ADD(cpl_image_collapse_window_create)(const cpl_image * self,
             pi += self->nx;
             if (bpmi != NULL) bpmi += self->nx;
         }
+    } else {
+        (void)cpl_error_set_(CPL_ERROR_ILLEGAL_INPUT);
+        other = NULL;
     }
 
 #ifdef CPL_TYPE_IS_FPOINT
@@ -1584,7 +1562,7 @@ CPL_TYPE_ADD(cpl_image_collapse_window_create)(const cpl_image * self,
                        the order of the pixels needs to be reversed */
   
                     /* Flip the 1D-image and swap nx and ny */
-                    CPL_TYPE_ADD(cpl_image_flip)(pi, (size_t)self->nx);
+                    CPL_TYPE_ADD(cpl_tools_flip)(pi, (size_t)self->nx);
                     self->ny = self->nx;
                     self->nx = 1;
                 } else {
@@ -1610,7 +1588,7 @@ CPL_TYPE_ADD(cpl_image_collapse_window_create)(const cpl_image * self,
                 /* A 180 degree rotation preserves the shape of the image,
                    and reverts the order of the pixels */
 
-                CPL_TYPE_ADD(cpl_image_flip)((CPL_TYPE *)self->pixels,
+                CPL_TYPE_ADD(cpl_tools_flip)((CPL_TYPE *)self->pixels,
                                              (size_t)self->nx *
                                              (size_t)self->ny);
                 break;
@@ -1645,7 +1623,7 @@ CPL_TYPE_ADD(cpl_image_collapse_window_create)(const cpl_image * self,
                        the order of the pixels needs to be reversed */
 
                     /* Flip the 1D-image and swap nx and ny */
-                    CPL_TYPE_ADD(cpl_image_flip)(pi, (size_t)self->ny);
+                    CPL_TYPE_ADD(cpl_tools_flip)(pi, (size_t)self->ny);
                     self->nx = self->ny;
                     self->ny = 1;
                 } else {
@@ -1699,7 +1677,7 @@ CPL_TYPE_ADD(cpl_image_collapse_window_create)(const cpl_image * self,
         case 2: {
 
             for (j = 0; j < ny; j++, pi += nx) {
-                CPL_TYPE_ADD(cpl_image_flip)(pi, nx);
+                CPL_TYPE_ADD(cpl_tools_flip)(pi, nx);
             }
             break;
         }
