@@ -1,6 +1,6 @@
 /*
  * This file is part of the ESO Common Pipeline Library
- * Copyright (C) 2001-2014 European Southern Observatory
+ * Copyright (C) 2001-2017 European Southern Observatory
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -38,6 +38,7 @@
 
 #include <math.h>
 #include <string.h>
+#include <assert.h>
 
 /*-----------------------------------------------------------------------------
                             Private Function prototypes
@@ -889,6 +890,8 @@ static int g2efunc2(int ndata, int npar, double *p, double *deviates,
     float * dyda = cpl_malloc(npar * sizeof(*dyda));
     float fp[MA];
 
+    assert(d == NULL);
+
     for (j = 0; j < MA; j++) {
         fp[j] = p[j];
     }
