@@ -414,11 +414,14 @@ void cpl_imagelist_empty(cpl_imagelist * self)
                 if (bpm != NULL &&
                     bpm == cpl_image_get_bpm_const(self->images[i])) {
                     /* The same bad pixel map is used also by this image */
-                    (void)cpl_image_unset_bpm(self->images[i]);
+                    const cpl_mask * samebpm
+                        = cpl_image_unset_bpm(self->images[i]);
+                    assert(samebpm == bpm);
                 }
                if (self->images[i] == del) {
                     /* This image was inserted more than once in the list */
-                    (void)cpl_imagelist_unset(self, i);
+                   const cpl_image * sameimg = cpl_imagelist_unset(self, i);
+                   assert(sameimg == del);
                 }
             }
         }
@@ -702,6 +705,10 @@ cpl_error_code cpl_imagelist_erase(
   CPL_IO_APPEND (append a list of images to the last data unit, which
   must already contain compatible image(s)).
 
+  For the CPL_IO_APPEND mode it is recommended to pass a NULL pointer for the
+  output header, since updating the already existing header incurs significant
+  overhead.
+
   When the data written to disk are of an integer type, the output mode
   CPL_IO_EXTEND can be combined (via bit-wise or) with an option for
   tile-compression. This compression of integer data is lossless.
