@@ -1,7 +1,6 @@
-/* $Id: cpl_io_fits.c,v 1.77 2012-12-10 13:20:33 llundin Exp $
- *
+/*
  * This file is part of the ESO Common Pipeline Library
- * Copyright (C) 2001-2008 European Southern Observatory
+ * Copyright (C) 2001-2017 European Southern Observatory
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,13 +17,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-/*
- * $Author: llundin $
- * $Date: 2012-12-10 13:20:33 $
- * $Revision: 1.77 $
- * $Name: not supported by cvs2svn $
- */
-
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -114,6 +106,8 @@ typedef struct cpl_fitsfile_t {
                         Private variables
  -----------------------------------------------------------------------------*/
 
+/* The maximum number of open FITS-files */
+static cpl_size cpl_io_max_open = (CPL_IO_FITS_MAX_OPEN);
 static cpl_size cpl_nfitsfiles = 0;    /* The number of open FITS-files */
 static cx_list * cpl_fitslist  = NULL; /* The list of open, cached FITS-files */
 
@@ -121,7 +115,7 @@ static cx_list * cpl_fitslist  = NULL; /* The list of open, cached FITS-files */
                                    Private functions
  -----------------------------------------------------------------------------*/
 
-#if CPL_IO_FITS_MAX_OPEN > 0
+#ifdef CPL_IO_FITS
 static cpl_boolean cpl_io_fits_find_fptr(cx_list_iterator *, const char *,
                                          const int *, const struct stat *)
 #ifdef CPL_HAVE_ATTR_NONNULL
@@ -142,9 +136,9 @@ static void cpl_io_fits_set(fitsfile *, const char *, int, cpl_boolean)
     CPL_ATTR_NONNULL;
 
 static cpl_fitsfile_t * cpl_io_fits_unset_tid(int);
-static int cpl_io_fits_free(cpl_fitsfile_t *, int *)
+static int cpl_io_fits_free(cpl_fitsfile_t *, cpl_boolean, int *)
 #ifdef CPL_HAVE_ATTR_NONNULL
-    __attribute__((nonnull(2)))
+    __attribute__((nonnull(3)))
 #endif
     ;
 
@@ -161,15 +155,23 @@ static int cpl_io_fits_free(cpl_fitsfile_t *, int *)
  */
 void cpl_io_fits_init(void)
 {
-#ifdef CPL_IO_FITS_DEBUG
-    cpl_msg_debug(cpl_func, cpl_fitslist == NULL
-                  ? "Initializing, max file pointers: "
-                  CPL_STRINGIFY(CPL_IO_FITS_MAX_OPEN)
-                  : "Already initialized, max file pointers: "
-                  CPL_STRINGIFY(CPL_IO_FITS_MAX_OPEN));
+#if defined HAVE_SYSCONF && defined _SC_OPEN_MAX
+    const long open_max = sysconf(_SC_OPEN_MAX);
+
+    if (0 <= open_max && open_max/2 < (CPL_IO_FITS_MAX_OPEN))
+        cpl_io_max_open = (cpl_size)(open_max/2);
 #endif
 
-#if CPL_IO_FITS_MAX_OPEN > 0
+#ifdef CPL_IO_FITS_DEBUG
+    cpl_msg_debug(cpl_func, cpl_fitslist == NULL
+                  ? "Initializing, max file pointers: %" CPL_SIZE_FORMAT " <= "
+                  CPL_STRINGIFY(CPL_IO_FITS_MAX_OPEN)
+                  : "Already initialized, max file pointers: %"
+                  CPL_SIZE_FORMAT " <= "
+                  CPL_STRINGIFY(CPL_IO_FITS_MAX_OPEN), cpl_io_max_open);
+#endif
+
+#ifdef CPL_IO_FITS
 #ifdef _OPENMP
 #pragma omp critical(cpl_io_fits)
 #endif
@@ -198,7 +200,7 @@ cpl_error_code cpl_io_fits_end(void)
     cpl_msg_debug(cpl_func, "Finished: " CPL_STRINGIFY(CPL_IO_FITS_MAX_OPEN));
 #endif
 
-#if CPL_IO_FITS_MAX_OPEN > 0
+#ifdef CPL_IO_FITS
 
 #ifdef _OPENMP
 #pragma omp critical(cpl_io_fits)
@@ -238,7 +240,8 @@ cpl_error_code cpl_io_fits_close_tid(cpl_boolean mode)
             oldest = cpl_io_fits_unset_tid(tid);
 
             /* If a matching file is found, close it */
-        } while (oldest != NULL && !cpl_io_fits_free(oldest, &status));
+        } while (oldest != NULL &&
+                 !cpl_io_fits_free(oldest, CPL_FALSE, &status));
     }
 
     return status ? cpl_error_set_where_() : CPL_ERROR_NONE;
@@ -307,12 +310,12 @@ int cpl_io_fits_create_file(fitsfile **pfptr, const char *filename, int *status)
                        exceeding the limit */
 
                     cpl_nfitsfiles++;
-                    if (cpl_nfitsfiles > (CPL_IO_FITS_MAX_OPEN)) {
+                    if (cpl_nfitsfiles > cpl_io_max_open) {
                         /* First need to close a file - find it first */
                         oldest = cpl_io_fits_unset_tid(tid);
                     }
                 }
-                if (cpl_io_fits_free(oldest, status)) {
+                if (cpl_io_fits_free(oldest, CPL_FALSE, status)) {
 #ifdef _OPENMP
 #pragma omp atomic
 #endif
@@ -323,6 +326,9 @@ int cpl_io_fits_create_file(fitsfile **pfptr, const char *filename, int *status)
         }
 
         if (fits_create_file(pfptr, filename, status)) {
+            (void)cpl_error_set_fits(CPL_ERROR_FILE_NOT_CREATED, *status,
+                                     fits_create_file, "filename='%s'",
+                                     filename);
 #ifdef _OPENMP
 #pragma omp atomic
 #endif
@@ -426,6 +432,11 @@ int cpl_io_fits_open_diskfile(fitsfile **pfptr, const char * filename,
 #ifdef CPL_IO_FITS_REWIND
 	    /* A newly opened file points to the 1st HDU so do the same here */
 	    if (fits_movabs_hdu(*pfptr, 1, NULL, status)) {
+                (void)cpl_error_fits(iomode == READWRITE
+                                     ? CPL_ERROR_FILE_NOT_CREATED
+                                     : CPL_ERROR_FILE_NOT_FOUND, status,
+                                     fits_movabs_hdu, "filename='%s', mode=%d",
+                                     filename, iomode);
 #ifdef CPL_IO_FITS_DEBUG
                 cpl_msg_debug(cpl_func, "Could not move to primary HDU: %s (%p) (I/O-"
                               "mode: %d) (%d)", filename, (const void*)*pfptr,
@@ -451,12 +462,12 @@ int cpl_io_fits_open_diskfile(fitsfile **pfptr, const char * filename,
                     below the limit and then the all try to open, thus 
                     exceeding the limit */
                 cpl_nfitsfiles++;
-                if (cpl_nfitsfiles > (CPL_IO_FITS_MAX_OPEN)) {
+                if (cpl_nfitsfiles > cpl_io_max_open) {
                     /* First need to close a file - find it first */
                     oldest = cpl_io_fits_unset_tid(tid);
                 }
             }
-            if (cpl_io_fits_free(oldest, status)) {
+            if (cpl_io_fits_free(oldest, CPL_FALSE, status)) {
 #ifdef _OPENMP
 #pragma omp atomic
 #endif
@@ -470,6 +481,12 @@ int cpl_io_fits_open_diskfile(fitsfile **pfptr, const char * filename,
                       filename, iomode, (int)cpl_nfitsfiles);
 #endif
         if (fits_open_diskfile(pfptr, filename, iomode, status)) {
+            (void)cpl_error_set_fits(iomode == READWRITE
+                                     ? CPL_ERROR_FILE_NOT_CREATED
+                                     : CPL_ERROR_FILE_NOT_FOUND, *status,
+                                     fits_open_diskfile,
+                                     "filename='%s', mode=%d", filename,
+                                     iomode);
 #ifdef _OPENMP
 #pragma omp atomic
 #endif
@@ -509,19 +526,85 @@ int cpl_io_fits_close_file(fitsfile *fptr, int *status)
 
         if (name == NULL) {
             /* This branch is used when CPL_IO_MODE is inactive */
-            (void)fits_close_file(fptr, status);
+            if (fits_close_file(fptr, status)) {
+                (void)cpl_error_set_fits(CPL_ERROR_BAD_FILE_FORMAT, *status,
+                                         fits_close_file, ".");
+            }
         } else if (iomode != READONLY) {
 #ifdef CPL_IO_FITS_DEBUG
             cpl_msg_debug(cpl_func, "Flushing handle (%p) for: %s (%d) (%d)",
                           (const void*)fptr, name, iomode, (int)cpl_nfitsfiles);
 #endif
-            (void)fits_flush_file(fptr, status);
+            if (fits_flush_file(fptr, status)) {
+                (void)cpl_error_set_fits(CPL_ERROR_BAD_FILE_FORMAT, *status,
+                                         fits_flush_file, "filename='%s', "
+                                         "mode=%d", name, iomode);
+            }
         }
     }
 
     return *status;
 }
 
+/**
+ * @internal
+ * @brief Wrapper around fits_delete_file (used on error), with a close first
+ * @param fptr    CFITSIO file pointer to file
+ * @param status  Pointer to CFITSIO error status
+ * @return        The CFITSIO error status
+ * @see fits_delete_file(), cpl_io_fits_close_file
+ * @note This wrapper is needed when CPL_IO_MODE is active
+
+ */
+int cpl_io_fits_delete_file(fitsfile *fptr, int *status)
+{
+
+    if (fptr != NULL) { /* Duplicate CFITSIO behaviour (*status ignored) */
+        int          iomode = 0; /* Initialize, in case the find fails */
+        const char * name = cpl_io_fits_find_name(fptr, &iomode);
+
+        cpl_fitsfile_t * cpl_fitsfile = NULL;
+        const char * filename = name != NULL && *name == '!' ? name+1 : name;
+        cx_list_iterator pos;
+        struct stat statbuf;
+        const cpl_boolean has_stat = filename ? !stat(filename, &statbuf)
+            : CPL_FALSE;
+
+        if (name != NULL) {
+            /* CPL_IO_MODE is active */
+
+#ifdef _OPENMP
+#pragma omp critical(cpl_io_fits)
+#endif
+            {
+                if (cpl_io_fits_find_fptr(&pos, filename, &iomode,
+                                          has_stat ? &statbuf : NULL)) {
+                    /* Found it */
+
+                    cpl_fitsfile = (cpl_fitsfile_t *)cx_list_extract(cpl_fitslist,
+                                                                     pos);
+                }
+            }
+        }
+
+        if (cpl_fitsfile != NULL) {
+            int status_ = 0;
+            cpl_io_fits_free(cpl_fitsfile, CPL_TRUE, &status_);
+            *status = status_;
+        } else {
+            int status_ = 0;
+            if (fits_delete_file(fptr, &status_)) {
+                (void)cpl_error_set_fits(CPL_ERROR_BAD_FILE_FORMAT, status_,
+                                         fits_delete_file, "filename='%s', "
+                                         "I/O-mode: %d <=> %d/%d",
+                                         name, iomode, READONLY, READWRITE);
+            }
+            *status = status_;
+        }
+    }
+
+    return *status;
+}
 
 /**
  * @internal
@@ -534,7 +617,7 @@ int cpl_io_fits_close_file(fitsfile *fptr, int *status)
 static cpl_fitsfile_t * cpl_io_fits_unset_tid(int tid)
 {
 
-#if CPL_IO_FITS_MAX_OPEN > 0
+#ifdef CPL_IO_FITS
 
     cx_list_iterator pos = cx_list_begin(cpl_fitslist);
 
@@ -558,22 +641,35 @@ static cpl_fitsfile_t * cpl_io_fits_unset_tid(int tid)
 
 /**
  * @internal
- * @brief Deallocate one pointer structure, closing the CFITS file
+ * @brief Deallocate one pointer structure, closing or deleting the CFITS file
    @param self    The pointer structure to deallocate, or NULL
+   @param dodel   Iff true then delete instead of just closing the file
    @param status  The CFITSIO status
    @return Zero on success or else the CFITSIO status
  */
-static int cpl_io_fits_free(cpl_fitsfile_t * self, int * status)
+static
+int cpl_io_fits_free(cpl_fitsfile_t * self, cpl_boolean dodel, int * status)
 {
 
     if (self != NULL) {
         if (*status == 0) {
-            if (fits_close_file(self->fptr, status)) {
+            if (dodel) {
+                if (fits_delete_file(self->fptr, status)) {
+                    (void)cpl_error_set_fits(CPL_ERROR_BAD_FILE_FORMAT, *status,
+                                             fits_delete_file, "filename='%s', "
+                                             "I/O-mode: %d, Thread-ID: %d",
+                                             self->name, self->iomode,
+                                             self->tid);
+                }
+            } else if (fits_close_file(self->fptr, status)) {
                 (void)cpl_error_set_fits(CPL_ERROR_BAD_FILE_FORMAT, *status,
                                          fits_close_file, "filename='%s', "
                                          "I/O-mode: %d, Thread-ID: %d",
-                                         self->name, self->iomode, self->tid);
-            } else {
+                                         self->name, self->iomode,
+                                         self->tid);
+            }
+
+            if (*status == 0) {
 #ifdef _OPENMP
 #pragma omp atomic
 #endif
@@ -644,7 +740,7 @@ int cpl_io_fits_close(const char * filename, int * status)
 static void cpl_io_fits_set(fitsfile * fptr, const char * name, int iomode,
                             cpl_boolean writing)
 {
-#if CPL_IO_FITS_MAX_OPEN > 0
+#ifdef CPL_IO_FITS
     if (cpl_fitslist != NULL) {
 
         char * filename = cpl_strdup(*name == '!' ? name+1 : name);
@@ -678,7 +774,7 @@ static void cpl_io_fits_set(fitsfile * fptr, const char * name, int iomode,
 #endif
 }
 
-#if CPL_IO_FITS_MAX_OPEN > 0
+#ifdef CPL_IO_FITS
 /*----------------------------------------------------------------------------*/
 /**
   @internal
@@ -766,7 +862,7 @@ static fitsfile * cpl_io_fits_unset_fptr(const char * name, const int * piomode)
 
     fitsfile * fptr = NULL;
 
-#if CPL_IO_FITS_MAX_OPEN > 0
+#ifdef CPL_IO_FITS
     if (cpl_fitslist != NULL) {
         cpl_fitsfile_t * cpl_fitsfile = NULL;
         const char * filename = name != NULL && *name == '!' ? name+1 : name;
@@ -824,7 +920,7 @@ static fitsfile * cpl_io_fits_reuse_fptr(const char * name, int iomode,
 
     fitsfile * fptr = NULL;
 
-#if CPL_IO_FITS_MAX_OPEN > 0
+#ifdef CPL_IO_FITS
     if (cpl_fitslist != NULL) {
         cpl_fitsfile_t * cpl_fitsfile = NULL;
         const char * filename = *name == '!' ? name+1 : name;
@@ -876,7 +972,7 @@ static const char * cpl_io_fits_find_name(const fitsfile * fptr, int * piomode)
 
     const char * name = NULL;
 
-#if CPL_IO_FITS_MAX_OPEN > 0
+#ifdef CPL_IO_FITS
 
     if (cpl_fitslist != NULL) {
 
