@@ -73,6 +73,10 @@ cpl_error_code cpl_polynomial_subtract(cpl_polynomial *,
                                        const cpl_polynomial *,
                                        const cpl_polynomial *);
 
+cpl_error_code cpl_polynomial_multiply(cpl_polynomial *,
+                                       const cpl_polynomial *,
+                                       const cpl_polynomial *);
+
 cpl_error_code cpl_polynomial_multiply_scalar(cpl_polynomial *,
                                               const cpl_polynomial *, double);
 
@@ -109,7 +113,8 @@ cpl_polynomial * cpl_polynomial_fit_1d_create(const cpl_vector *,
                                               double *) CPL_ATTR_DEPRECATED;
 
 /* Basic operations on 2d-polynomials */
-cpl_polynomial * cpl_polynomial_fit_2d_create(cpl_bivector *, cpl_vector *,
+cpl_polynomial * cpl_polynomial_fit_2d_create(const cpl_bivector *,
+                                              const cpl_vector *,
                                               cpl_size, double *)
     CPL_ATTR_DEPRECATED;
 
