@@ -28,10 +28,23 @@
 
 CPL_BEGIN_DECLS
 
+/*-----------------------------------------------------------------------------
+                             Macro definitions
+ -----------------------------------------------------------------------------*/
+
+#undef CPL_POLYNOMIAL_UNION_IS_DOUBLE_SZ
+#if defined(SIZEOF_VOID_P) && defined(SIZEOF_DOUBLE)
+#  if SIZEOF_VOID_P <= SIZEOF_DOUBLE
+#    define CPL_POLYNOMIAL_UNION_IS_DOUBLE_SZ
+#  endif
+#endif
+
 /*-----------------------------------------------------------------------------
                             Function prototypes
  -----------------------------------------------------------------------------*/
 
+void cpl_polynomial_empty(cpl_polynomial *);
+
 cpl_error_code cpl_polynomial_fit_1d(cpl_polynomial *, const cpl_vector *,
                                      const cpl_vector *, cpl_size, cpl_size,
                                      cpl_boolean, double *);
@@ -42,6 +55,12 @@ void cpl_polynomial_shift_double(double *, cpl_size, double)
 cpl_error_code cpl_polynomial_solve_1d_(const cpl_polynomial *, double,
                                         double *, cpl_size, cpl_boolean);
 
+cpl_size cpl_polynomial_get_1d_size(const cpl_polynomial *)
+    CPL_ATTR_PURE CPL_ATTR_NONNULL;
+
+double cpl_polynomial_eval_monomial(const cpl_polynomial *,
+                                    const cpl_vector *);
+
 CPL_END_DECLS
 
 #endif 
