@@ -1,7 +1,6 @@
-/* $Id: cpl_type_impl.h,v 1.3 2011-02-28 14:21:38 cizzo Exp $
- *
+/*
  * This file is part of the ESO Common Pipeline Library
- * Copyright (C) 2001-2005 European Southern Observatory
+ * Copyright (C) 2001-2017 European Southern Observatory
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,13 +17,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-/*
- * $Author: cizzo $
- * $Date: 2011-02-28 14:21:38 $
- * $Revision: 1.3 $
- * $Name: not supported by cvs2svn $
- */
-
 #ifndef CPL_TYPE_IMPL_H
 #define CPL_TYPE_IMPL_H
 
@@ -44,5 +36,7 @@ CPL_BEGIN_DECLS
  *       a 64-bit signed integer type
  */
 
+CPL_END_DECLS
+
 #endif
 /* end of cpl_type_impl.h */
