@@ -1,7 +1,6 @@
-/* $Id: cpl_vector.h,v 1.66 2013-02-04 12:57:00 llundin Exp $
- *
+/*
  * This file is part of the ESO Common Pipeline Library
- * Copyright (C) 2001-2008 European Southern Observatory
+ * Copyright (C) 2001-2017 European Southern Observatory
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,13 +17,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-/*
- * $Author: llundin $
- * $Date: 2013-02-04 12:57:00 $
- * $Revision: 1.66 $
- * $Name: not supported by cvs2svn $
- */
-
 #ifndef CPL_VECTOR_H
 #define CPL_VECTOR_H
 
@@ -141,6 +133,7 @@ cpl_error_code cpl_vector_subtract(cpl_vector *, const cpl_vector *);
 cpl_error_code cpl_vector_multiply(cpl_vector *, const cpl_vector *);
 cpl_error_code cpl_vector_divide(cpl_vector *, const cpl_vector *);
 double cpl_vector_product(const cpl_vector *, const cpl_vector *);
+cpl_error_code cpl_vector_cycle(cpl_vector *, const cpl_vector *, double);
 cpl_error_code cpl_vector_sort(cpl_vector *, cpl_sort_direction);
 cpl_error_code cpl_vector_add_scalar(cpl_vector *, double);
 cpl_error_code cpl_vector_subtract_scalar(cpl_vector *, double);
@@ -157,6 +150,8 @@ cpl_vector * cpl_vector_extract(const cpl_vector *, cpl_size,
                                 cpl_size, cpl_size) CPL_ATTR_ALLOC;
 
 /* Statistics on cpl_vector */
+cpl_size cpl_vector_get_minpos(const cpl_vector *);
+cpl_size cpl_vector_get_maxpos(const cpl_vector *);
 double cpl_vector_get_min(const cpl_vector *);
 double cpl_vector_get_max(const cpl_vector *);
 double cpl_vector_get_sum(const cpl_vector *);
