@@ -26,5 +26,31 @@ CPL_BEGIN_DECLS
 
 double * cpl_vector_rewrap(cpl_vector *, cpl_size, double *);
 
+cpl_size cpl_vector_get_size_(const cpl_vector *)
+    CPL_ATTR_PURE CPL_ATTR_NONNULL;
+
+double * cpl_vector_get_data_(cpl_vector *)
+#ifdef NDEBUG
+    CPL_ATTR_PURE
+#endif
+    CPL_ATTR_NONNULL;
+
+const double * cpl_vector_get_data_const_(const cpl_vector *)
+#ifdef NDEBUG
+    CPL_ATTR_PURE
+#endif
+    CPL_ATTR_NONNULL;
+
+double cpl_vector_get_(const cpl_vector *, cpl_size)
+#ifdef NDEBUG
+    CPL_ATTR_PURE
+#endif
+    CPL_ATTR_NONNULL;
+
+void cpl_vector_set_(cpl_vector *,
+                     cpl_size,
+                     double)
+    CPL_ATTR_NONNULL;
+
 #endif
 /* end of cpl_vector_impl.h */
