@@ -245,7 +245,10 @@ static void cpl_image_bpm_reject_value_test(void)
         cpl_test_eq(nbad, IMAGESZ * IMAGESZ);
 
         /* Various errors */
-        code = cpl_image_reject_value(img, CPL_VALUE_ZERO || CPL_VALUE_NAN);
+        code = cpl_image_reject_value(img, 0); /* Result of binary and */
+        cpl_test_eq_error(code, CPL_ERROR_UNSUPPORTED_MODE);
+
+        code = cpl_image_reject_value(img, 1); /* Result of logical and/or */
         cpl_test_eq_error(code, CPL_ERROR_INVALID_TYPE);
 
         code = cpl_image_reject_value(img, ~CPL_VALUE_NOTFINITE);
