@@ -1,8 +1,6 @@
-
-/* $Id: cpl_io_fits-test.c,v 1.14 2012-05-02 11:02:32 llundin Exp $
- *
+/*
  * This file is part of the ESO Common Pipeline Library
- * Copyright (C) 2001-2008 European Southern Observatory
+ * Copyright (C) 2001-2017 European Southern Observatory
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,27 +17,21 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-/*
- * $Author: llundin $
- * $Date: 2012-05-02 11:02:32 $
- * $Revision: 1.14 $
- * $Name: not supported by cvs2svn $
- */
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 /*-----------------------------------------------------------------------------
                                    Includes
  -----------------------------------------------------------------------------*/
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include "cpl_io_fits.h"
 
 #include "cpl_test.h"
 
 #include "cpl_fits.h"
 #include "cpl_memory.h"
+#include "cpl_table.h"
 
 #include <stdlib.h>
 #include <string.h>
@@ -49,7 +41,7 @@
  -----------------------------------------------------------------------------*/
 
 #ifndef NEXT
-#define NEXT 17
+#define NEXT 3
 #endif
 
 #ifndef NFILE
@@ -63,11 +55,12 @@
  ----------------------------------------------------------------------------*/
 
 static void cpl_io_fits_test_many(int);
+static void cpl_io_fits_test_fulldisk_table(const char*);
 
 /*-----------------------------------------------------------------------------
                                   Main
  -----------------------------------------------------------------------------*/
-int main(void)
+int main(int argc, char *argv[])
 {
     cpl_boolean    do_bench;
     int            next = NEXT;
@@ -84,6 +77,10 @@ int main(void)
 
     /* Insert tests below */
 
+    if (argc == 2) {
+        cpl_io_fits_test_fulldisk_table(argv[1]);
+    }
+
     cpl_io_fits_test_many(1 + CPL_IO_FITS_MAX_OPEN);
 
 #ifdef _OPENMP
@@ -396,6 +393,18 @@ int main(void)
         cpl_free((void*)filename);
     }
 
+    if (cpl_fits_get_mode() | CPL_FITS_START_CACHING) {
+        cpl_error_code myerror;
+        myerror = cpl_fits_set_mode(CPL_FITS_STOP_CACHING);
+        cpl_test_eq_error(myerror, CPL_ERROR_NONE);
+    }
+
+    for (j = 0; j < NFILE; j++) {
+        char * filename = cpl_sprintf(BASENAME "-%d.fits", j + 1);
+        cpl_test_zero(remove(filename));
+        cpl_free(filename);
+    }
+
     /* End of tests */
     return cpl_test_end(0);
 }
@@ -411,10 +420,13 @@ int main(void)
 static void cpl_io_fits_test_many(int n)
 {
 
-    int i;
     cpl_propertylist * plist = cpl_propertylist_new();
+    const cpl_fits_mode iomode = cpl_fits_get_mode();
 
-    for (i = 0; i< n; i++) {
+    cpl_msg_info(cpl_func, "Testing I/O-mode %d <=> %d with %d file(s)",
+                 (int)iomode, (int)CPL_FITS_START_CACHING, n);
+
+    for (int i = 0; i < n; i++) {
         char * filename = cpl_sprintf(BASENAME "-%d.fits", i + 1);
         cpl_error_code myerror;
         myerror = cpl_propertylist_update_int(plist, "MYINT", i);
@@ -436,4 +448,82 @@ static void cpl_io_fits_test_many(int n)
 
     cpl_propertylist_delete(plist);
 
+    if (iomode | CPL_FITS_START_CACHING) {
+        cpl_error_code myerror;
+        myerror = cpl_fits_set_mode(CPL_FITS_STOP_CACHING);
+        cpl_test_eq_error(myerror, CPL_ERROR_NONE);
+    }
+
+    for (int i = 0; i < n; i++) {
+        char * filename = cpl_sprintf(BASENAME "-%d.fits", i + 1);
+        cpl_test_zero(remove(filename));
+        cpl_free(filename);
+    }
+    if (iomode | CPL_FITS_START_CACHING) {
+        cpl_error_code myerror;
+        myerror = cpl_fits_set_mode(iomode);
+        cpl_test_eq_error(myerror, CPL_ERROR_NONE);
+    }
+}
+
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief       Test I/O on a user specified file system (of limited size)
+  @param  root The name of the file system to fill up
+  @return void
+ */
+/*----------------------------------------------------------------------------*/
+static void cpl_io_fits_test_fulldisk_table(const char* root)
+{
+    cpl_table*     mytab = cpl_table_new(1000);
+    char*          filename = cpl_sprintf("%s/" BASENAME ".fits", root);
+    const cpl_fits_mode iomode = cpl_fits_get_mode();
+    cpl_error_code code;
+
+    cpl_test_nonnull(mytab);
+    cpl_test_nonnull(filename);
+
+    code = cpl_table_new_column(mytab, "COL1", CPL_TYPE_DOUBLE);
+    cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+    code = cpl_table_fill_invalid_double(mytab, "COL1", 42.0);
+    cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+    code = cpl_table_save(mytab, NULL, NULL, filename, CPL_IO_CREATE);
+    cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+    cpl_test_fits(filename);
+
+    do {
+        code = cpl_table_save(mytab, NULL, NULL, filename, CPL_IO_EXTEND);
+    } while (code == CPL_ERROR_NONE);
+
+    cpl_test_eq_error(code, CPL_ERROR_FILE_NOT_CREATED);
+
+    if (cpl_test_get_failed() > 0) {
+        cpl_msg_warning(cpl_func, "Test failed, not deleting file:: %s",
+                        filename);
+    } else if (remove(filename) != 0) {
+        cpl_msg_info(cpl_func, "The failure-test file has already been deleted: "
+                     "%s", filename);
+    }
+
+    cpl_table_delete(mytab);
+
+    if (iomode | CPL_FITS_START_CACHING) {
+        cpl_error_code myerror;
+        myerror = cpl_fits_set_mode(CPL_FITS_STOP_CACHING);
+        cpl_test_eq_error(myerror, CPL_ERROR_NONE);
+    }
+
+    cpl_test_zero(remove(filename));
+
+    if (iomode | CPL_FITS_START_CACHING) {
+        cpl_error_code myerror;
+        myerror = cpl_fits_set_mode(iomode);
+        cpl_test_eq_error(myerror, CPL_ERROR_NONE);
+    }
+
+    cpl_free(filename);
 }
