@@ -1,7 +1,6 @@
-/* $Id: cpl_math-test.c,v 1.12 2010-11-11 09:23:18 llundin Exp $
- *
+/*
  * This file is part of the ESO Common Pipeline Library
- * Copyright (C) 2001-2008 European Southern Observatory
+ * Copyright (C) 2001-2017 European Southern Observatory
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,21 +17,14 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-/*
- * $Author: llundin $
- * $Date: 2010-11-11 09:23:18 $
- * $Revision: 1.12 $
- * $Name: not supported by cvs2svn $
- */
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 /*-----------------------------------------------------------------------------
                                    Includes
  -----------------------------------------------------------------------------*/
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include <math.h>
 #include <float.h>
 
@@ -59,9 +51,6 @@ int main(void)
 
     /* Insert tests below */
 
-    /* The constants are of type double */
-    cpl_test_eq(sizeof(CPL_MATH_PI), sizeof(double));
-
     /* Expressions with just a subtraction must be accurate to 1 unit
        of machine precision, expressions also involving division must be
        accurate to 2 units of machine precision */
