@@ -1,7 +1,6 @@
-/* $Id: cpl_plot-test.c,v 1.17 2010-11-11 09:23:18 llundin Exp $
- *
+/*
  * This file is part of the ESO Common Pipeline Library
- * Copyright (C) 2001-2008 European Southern Observatory
+ * Copyright (C) 2001-2017 European Southern Observatory
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,21 +17,14 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-/*
- * $Author: llundin $
- * $Date: 2010-11-11 09:23:18 $
- * $Revision: 1.17 $
- * $Name: not supported by cvs2svn $
- */
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 /*-----------------------------------------------------------------------------
                                    Includes
  -----------------------------------------------------------------------------*/
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #ifdef HAVE_SETENV
 extern int setenv(const char *, const char *, int);
 #endif
@@ -59,12 +51,14 @@ extern int setenv(const char *, const char *, int);
                                    Private functions
  -----------------------------------------------------------------------------*/
 
+static
 void cpl_plot_test_all(int, int, int);
 
+static
 void cpl_plot_test(const cpl_image *, const cpl_mask *, const cpl_table *,
                    int, const char *[],
                    const cpl_vector *, const cpl_bivector *,
-                   int, const cpl_vector **);
+                   int, const cpl_vector * const *);
 
 
 /*-----------------------------------------------------------------------------
@@ -95,6 +89,7 @@ int main(void)
   
  */
 /*----------------------------------------------------------------------------*/
+static
 void cpl_plot_test_all(int n2d, int nvec, int nrow)
 {
 
@@ -152,8 +147,8 @@ void cpl_plot_test_all(int n2d, int nvec, int nrow)
     cpl_table_new_column(tab, names[2], CPL_TYPE_DOUBLE);
     for (i=0; i < nrow; i++) {
         cpl_table_set_double(tab, names[0], i, (double)(10*i+1));
-        cpl_table_set_double(tab, names[1], i, sin(10*i+1)/10*i+1);
-        cpl_table_set_double(tab, names[2], i, log1p(i)/i+1);
+        cpl_table_set_double(tab, names[1], i, sin(10*i+1)/(10*i+1));
+        cpl_table_set_double(tab, names[2], i, log1p(i)/(i+1));
     }
 
     /* No errors should have happened in the plot preparation */
@@ -163,21 +158,21 @@ void cpl_plot_test_all(int n2d, int nvec, int nrow)
     if (cpl_msg_get_level() <= CPL_MSG_INFO) {
 
         cpl_plot_test(ima, mask, tab, 3, names, sinus, sin_deg, 3,
-                      (const cpl_vector**)vec_array);
+                      (const cpl_vector* const *)vec_array);
 
 #ifdef HAVE_SETENV
         if (!setenv("CPL_IMAGER", "display - &", 1) &&
             !setenv("CPL_PLOTTER", "cat > /dev/null; echo gnuplot > /dev/null",
                     1)) {
             cpl_plot_test(ima, mask, tab, 3, names, sinus, sin_deg, 3,
-                          (const cpl_vector**)vec_array);
+                          (const cpl_vector* const *)vec_array);
         }
     } else {
         if (!setenv("CPL_IMAGER", "cat > /dev/null", 1) &&
             !setenv("CPL_PLOTTER", "cat > /dev/null; echo gnuplot > /dev/null",
                     1)) {
             cpl_plot_test(ima, mask, tab, 3, names, sinus, sin_deg, 3,
-                          (const cpl_vector**)vec_array);
+                          (const cpl_vector* const *)vec_array);
         }
 #endif
     }
@@ -211,10 +206,11 @@ void cpl_plot_test_all(int n2d, int nvec, int nrow)
   
  */
 /*----------------------------------------------------------------------------*/
+static
 void cpl_plot_test(const cpl_image * image, const cpl_mask * mask,
                    const cpl_table * table, int ncolumns, const char *names[],
                    const cpl_vector * vector, const cpl_bivector * bivector,
-                   int nvector, const cpl_vector **vec_array)
+                   int nvector, const cpl_vector * const *vec_array)
 {
 
     /* An error may occur, e.g. in the absence of gnuplot.
@@ -236,8 +232,10 @@ void cpl_plot_test(const cpl_image * image, const cpl_mask * mask,
     error = cpl_plot_vector("", "w lines", "", vector);
     cpl_test_eq_error(error, error0);
 
+    CPL_DIAG_PRAGMA_PUSH_IGN(-Wcast-qual);
     error = cpl_plot_vectors("", "w lines", "",
-                             vec_array, nvector);
+                             (const cpl_vector**)vec_array, nvector);
+    CPL_DIAG_PRAGMA_POP;
     cpl_test_eq_error(error, error0);
 
     error = cpl_plot_image("", "w lines", "", image);
