@@ -38,6 +38,7 @@
 #include <stdio.h>
 #include <math.h>
 #include <float.h>
+#include <assert.h>
 
 /* Needed by memcpy() */
 #include <string.h>
@@ -101,6 +102,9 @@ struct _cpl_apertures_ {
  -----------------------------------------------------------------------------*/
 
 static cpl_apertures * cpl_apertures_new(cpl_size) CPL_ATTR_ALLOC;
+static cpl_apertures * cpl_apertures_new_from_image_(const cpl_image *,
+                                                     const cpl_image *,
+                                                     cpl_size, cpl_boolean);
 
 /*-----------------------------------------------------------------------------
                             Function codes
@@ -236,227 +240,12 @@ void cpl_apertures_dump(const cpl_apertures * self,
 cpl_apertures * cpl_apertures_new_from_image(const cpl_image * self,
                                              const cpl_image * lab)
 {
-    cpl_apertures   *   aperts;
-    const cpl_image *   dimage;
-    cpl_image       *   cimage = NULL;
-    const cpl_mask  *   bpm;
-    cpl_image       *   labcopy = NULL;
-    const cpl_image *   labuse;
-    cpl_size            naperts;
-    const double    *   pd;
-    const int       *   plab;
-    double          *   sqsum;
-    cpl_size            npix;
-    double          *   med_array;
-    double              weight;
-    cpl_size            count;
-    const cpl_size      nx = cpl_image_get_size_x(self);
-    const cpl_size      ny = cpl_image_get_size_y(self);
-    const double        ep = 1e-10;
-    cpl_size            i, j, k;
 
-    /* Test entries */
-    cpl_ensure(self != NULL, CPL_ERROR_NULL_INPUT, NULL);
-    cpl_ensure(lab  != NULL, CPL_ERROR_NULL_INPUT, NULL);
-    cpl_ensure(cpl_image_get_type(lab) == CPL_TYPE_INT,
-               CPL_ERROR_TYPE_MISMATCH, NULL);
-    cpl_ensure(nx == cpl_image_get_size_x(lab),
-               CPL_ERROR_INCOMPATIBLE_INPUT, NULL);
-    cpl_ensure(ny == cpl_image_get_size_y(lab),
-               CPL_ERROR_INCOMPATIBLE_INPUT, NULL);
+    cpl_apertures * aperts = cpl_apertures_new_from_image_(self, lab, 0,
+                                                           CPL_TRUE);
 
-    bpm = cpl_image_get_bpm_const(self);
-    if (bpm != NULL && !cpl_mask_is_empty(bpm)) {
-        /* Set the bad pixels of the input image to 0 in the lab image */
-        /* FIXME: Is this even necessary ? */
+    if (aperts == NULL) (void)cpl_error_set_where_();
 
-        labcopy = cpl_image_duplicate(lab);
-        cpl_image_reject_from_mask(labcopy, bpm);
-        cpl_image_fill_rejected(labcopy, 0.0);
-    }
-    labuse = labcopy ? labcopy : lab;
-
-    plab = cpl_image_get_data_int_const(labuse);
-
-    /* Get number of apertures from the labelised image */
-    naperts = (cpl_size)cpl_image_get_max(labuse);
-    if (naperts <= 0) {
-        cpl_image_delete(labcopy);
-        (void)cpl_error_set_(CPL_ERROR_ILLEGAL_INPUT);
-        return NULL;
-    }
-
-    /* Convert input image in a double image */
-    dimage = cpl_image_get_type(self) == CPL_TYPE_DOUBLE ? self
-        : (cimage = cpl_image_cast(self, CPL_TYPE_DOUBLE));
-
-    if (dimage == NULL) {
-        cpl_image_delete(labcopy);
-        cpl_image_delete(cimage);
-        (void)cpl_error_set_where_();
-        return NULL;
-    }
-
-    pd = cpl_image_get_data_double_const(dimage);
-
-    /* Create a cpl_apertures */
-    aperts = cpl_apertures_new(naperts);
-    sqsum  = cpl_calloc((size_t)naperts, sizeof(double));
-
-    /* Initialise min and max */
-    for (k = 0; k < naperts; k++) {
-        aperts->max_val[k] = -DBL_MAX;
-        aperts->min_val[k] = DBL_MAX;
-    }
-
-    /* Compute some stats */
-    for (j = 0; j < ny; j++) {
-        for (i = 0; i < nx; i++) {
-            double pix, delta;
-
-            k = plab[i+j*nx]-1;
-            /* Background: do nothing */
-            if (k==-1) continue;
-            if (k < 0) {
-                cpl_free(sqsum);
-                cpl_apertures_delete(aperts);
-                cpl_image_delete(cimage);
-                cpl_image_delete(labcopy);
-                (void)cpl_error_set_(CPL_ERROR_ILLEGAL_INPUT);
-                return NULL;
-            }
-
-            /* Current pixel value */
-            pix = pd[i+j*nx];
-
-            /* See cpl_tools_get_variance_double() */
-            delta = pix - aperts->mean[k];
-
-            aperts->varsum[k] += (double)aperts->npix[k] * delta * delta
-                / (double)(aperts->npix[k] + 1);
-            aperts->mean[k]   += delta / (double)(aperts->npix[k] + 1);
-
-            /* Accumulate positions */
-            aperts->x[k] += (double)(i+1);
-            aperts->y[k] += (double)(j+1);
-            /* Increase number of pixels */
-            aperts->npix[k] ++;
-            /* Store pixel sum and squared sum */
-            aperts->flux[k] += pix;
-            sqsum[k] += (pix * pix);
-            /* Check max pos value */
-            if (pix > aperts->max_val[k]) {
-                aperts->max_val[k] = pix;
-                aperts->maxpos_x[k] = i+1;
-                aperts->maxpos_y[k] = j+1;
-            }
-            /* Check min pos value */
-            if (pix < aperts->min_val[k]) {
-                aperts->min_val[k] = pix;
-                aperts->minpos_x[k] = i+1;
-                aperts->minpos_y[k] = j+1;
-            }
-
-            /* Check object extremities */
-            if ((i+1 < aperts->left_x[k]) || (aperts->left_x[k] == 0)) {
-                aperts->left_x[k] = i+1;
-                aperts->left_y[k] = j+1;
-            }
-            if (i+1 > aperts->right_x[k]) {
-                aperts->right_x[k] = i+1;
-                aperts->right_y[k] = j+1;
-            }
-            if ((j+1 < aperts->bottom_y[k]) || (aperts->bottom_y[k] == 0)) {
-                aperts->bottom_x[k] = i+1;
-                aperts->bottom_y[k] = j+1;
-            }
-            if (j+1 > aperts->top_y[k]) {
-                aperts->top_x[k] = i+1;
-                aperts->top_y[k] = j+1;
-            }
-        }
-    }
-
-    /* Centroiding */
-    for (j = 0; j < ny; j++) {
-        for (i = 0; i < nx; i++) {
-            double pix;
-
-            k = plab[i+j*nx]-1;
-            /* Background: do nothing */
-            if (k==-1) continue;
-            /* Current pixel value */
-            pix = pd[i+j*nx];
-            if (aperts->min_val[k] <= 0) {
-                /* Accumulate weighted positions */
-                aperts->xcentroid[k]+=(double)(i+1)*(pix-aperts->min_val[k]+ep);
-                aperts->ycentroid[k]+=(double)(j+1)*(pix-aperts->min_val[k]+ep);
-            } else {
-                /* Accumulate weighted positions */
-                aperts->xcentroid[k] += (double)(i+1) * pix;
-                aperts->ycentroid[k] += (double)(j+1) * pix;
-            }
-
-        }
-    }
-
-    /* Compute average and std dev for each aperture, normalize centers */
-    for (k = 0; k < aperts->naperts; k++) {
-        npix = aperts->npix[k];
-        if (npix<=0) {
-            cpl_free(sqsum);
-            cpl_apertures_delete(aperts);
-            cpl_image_delete(cimage);
-            cpl_image_delete(labcopy);
-            (void)cpl_error_set_(CPL_ERROR_DATA_NOT_FOUND);
-            return NULL;
-        }
-        if (npix > 1) {
-            aperts->stdev[k] = sqrt(aperts->varsum[k]/((double)npix-1.0));
-        } else {
-            aperts->stdev[k] = -1.0;
-        }
-        aperts->x[k] /= (double)npix;
-        aperts->y[k] /= (double)npix;
-        weight = aperts->flux[k];
-        if (aperts->min_val[k] <= 0) {
-            weight -= (double)npix * aperts->min_val[k];
-            weight += (double)npix * ep;
-        }
-
-        if (aperts->xcentroid[k] >= weight * (double)aperts->left_x[k] &&
-            aperts->xcentroid[k] <= weight * (double)aperts->right_x[k] &&
-            weight != 0.0) {
-            aperts->xcentroid[k] /= weight;
-        } else {
-            aperts->xcentroid[k] = 0.0;
-        }
-
-        if (aperts->ycentroid[k] >= weight * (double)aperts->bottom_y[k] &&
-            aperts->ycentroid[k] <= weight * (double)aperts->top_y[k] &&
-            weight != 0.0) {
-            aperts->ycentroid[k] /= weight;
-        } else {
-            aperts->ycentroid[k] = 0.0;
-        }
-    }
-    cpl_free(sqsum);
-
-    /* Compute median for each aperture */
-    for (k = 0; k < naperts; k++) {
-        med_array = cpl_malloc((size_t)aperts->npix[k] * sizeof(double));
-        count=0;
-        for (j = 0; j < ny; j++) {
-            for (i = 0; i < nx; i++) {
-                if (plab[i+j*nx]==(k+1)) med_array[count++] = pd[i+j*nx];
-            }
-        }
-        aperts->median[k] = cpl_tools_get_median_double(med_array, count);
-        cpl_free(med_array);
-    }
-
-    cpl_image_delete(cimage);
-    cpl_image_delete(labcopy);
     return aperts;
 }
 
@@ -1426,7 +1215,8 @@ cpl_apertures * cpl_apertures_extract_mask(const cpl_image * self,
     if (labels != NULL && nlabels > 0) {
 
         /* Create the detected apertures list */
-        aperts = cpl_apertures_new_from_image(self, labels);
+        aperts = cpl_apertures_new_from_image_(self, labels, nlabels,
+                                               CPL_FALSE);
 
     }
 
@@ -1628,3 +1418,271 @@ static cpl_apertures * cpl_apertures_new(cpl_size naperts)
     return aperts;
 }
 
+
+/*----------------------------------------------------------------------------*/
+/**
+  @internal
+  @brief  Compute statistics on selected apertures
+  @param  self      Reference image
+  @param  lab       Labelized image (of type CPL_TYPE_INT)
+  @param  maperts   Number of apertures, or zero for unknown
+  @param  bpm_check Iff true, any bpms must be deselected from the labels
+  @return  An CPL apertures object or @em NULL on error
+  @note The returned object must be deleted using cpl_apertures_delete().
+  @see cpl_apertures_new_from_image()
+
+ */
+/*----------------------------------------------------------------------------*/
+cpl_apertures * cpl_apertures_new_from_image_(const cpl_image * self,
+                                              const cpl_image * lab,
+                                              cpl_size          maperts,
+                                              cpl_boolean       bpm_check)
+{
+    cpl_apertures   *   aperts;
+    const cpl_image *   dimage;
+    cpl_image       *   cimage = NULL;
+    cpl_image       *   labcopy = NULL;
+    const cpl_image *   labuse;
+    cpl_size            naperts;
+    const double    *   pd;
+    const int       *   plab;
+    double          *   medians;
+    cpl_size        *   medianc;
+    const cpl_size      nx = cpl_image_get_size_x(self);
+    const cpl_size      ny = cpl_image_get_size_y(self);
+    const double        ep = 1e-10;
+    cpl_size            i, j, k;
+
+    /* Test entries */
+    cpl_ensure(self != NULL, CPL_ERROR_NULL_INPUT, NULL);
+    cpl_ensure(lab  != NULL, CPL_ERROR_NULL_INPUT, NULL);
+    cpl_ensure(cpl_image_get_type(lab) == CPL_TYPE_INT,
+               CPL_ERROR_TYPE_MISMATCH, NULL);
+    cpl_ensure(nx == cpl_image_get_size_x(lab),
+               CPL_ERROR_INCOMPATIBLE_INPUT, NULL);
+    cpl_ensure(ny == cpl_image_get_size_y(lab),
+               CPL_ERROR_INCOMPATIBLE_INPUT, NULL);
+
+    if (bpm_check) {
+        const cpl_mask * bpm = cpl_image_get_bpm_const(self);
+
+        if (bpm != NULL && !cpl_mask_is_empty(bpm)) {
+            /* Set the bad pixels of the input image to 0 in the lab image */
+            /* FIXME: Is this even necessary ? */
+
+            labcopy = cpl_image_duplicate(lab);
+            cpl_image_reject_from_mask(labcopy, bpm);
+            cpl_image_fill_rejected(labcopy, 0.0);
+        }
+    }
+    labuse = labcopy ? labcopy : lab;
+
+    plab = cpl_image_get_data_int_const(labuse);
+
+    /* Get number of apertures from the labelised image */
+    naperts = maperts != 0 ? maperts : (cpl_size)cpl_image_get_max(labuse);
+    if (naperts <= 0) {
+        cpl_image_delete(labcopy);
+        (void)cpl_error_set_(CPL_ERROR_ILLEGAL_INPUT);
+        return NULL;
+    }
+
+    /* Convert input image in a double image */
+    dimage = cpl_image_get_type(self) == CPL_TYPE_DOUBLE ? self
+        : (cimage = cpl_image_cast(self, CPL_TYPE_DOUBLE));
+
+    if (dimage == NULL) {
+        cpl_image_delete(labcopy);
+        cpl_image_delete(cimage);
+        (void)cpl_error_set_where_();
+        return NULL;
+    }
+
+    pd = cpl_image_get_data_double_const(dimage);
+
+    /* Create a cpl_apertures */
+    aperts = cpl_apertures_new(naperts);
+
+    /* Initialise min and max */
+    for (k = 0; k < naperts; k++) {
+        aperts->max_val[k] = -DBL_MAX;
+        aperts->min_val[k] = DBL_MAX;
+    }
+
+    /* Compute some stats */
+    for (j = 0; j < ny; j++) {
+        for (i = 0; i < nx; i++) {
+            double pix, delta;
+
+            k = plab[i+j*nx]-1;
+            /* Background: do nothing */
+            if (k==-1) continue;
+            if (k < 0) {
+                cpl_apertures_delete(aperts);
+                cpl_image_delete(cimage);
+                cpl_image_delete(labcopy);
+                (void)cpl_error_set_(CPL_ERROR_ILLEGAL_INPUT);
+                return NULL;
+            }
+
+            assert( k < naperts );
+
+            /* Current pixel value */
+            pix = pd[i+j*nx];
+
+            /* See cpl_tools_get_variance_double() */
+            delta = pix - aperts->mean[k];
+
+            aperts->varsum[k] += (double)aperts->npix[k] * delta * delta
+                / (double)(aperts->npix[k] + 1);
+            aperts->mean[k]   += delta / (double)(aperts->npix[k] + 1);
+
+            /* Accumulate positions */
+            aperts->x[k] += (double)(i+1);
+            aperts->y[k] += (double)(j+1);
+            /* Increase number of pixels */
+            aperts->npix[k] ++;
+            /* Store pixel sum and squared sum */
+            aperts->flux[k] += pix;
+            /* Check max pos value */
+            if (pix > aperts->max_val[k]) {
+                aperts->max_val[k] = pix;
+                aperts->maxpos_x[k] = i+1;
+                aperts->maxpos_y[k] = j+1;
+            }
+            /* Check min pos value */
+            if (pix < aperts->min_val[k]) {
+                aperts->min_val[k] = pix;
+                aperts->minpos_x[k] = i+1;
+                aperts->minpos_y[k] = j+1;
+            }
+
+            /* Check object extremities */
+            if ((i+1 < aperts->left_x[k]) || (aperts->left_x[k] == 0)) {
+                aperts->left_x[k] = i+1;
+                aperts->left_y[k] = j+1;
+            }
+            if (i+1 > aperts->right_x[k]) {
+                aperts->right_x[k] = i+1;
+                aperts->right_y[k] = j+1;
+            }
+            if ((j+1 < aperts->bottom_y[k]) || (aperts->bottom_y[k] == 0)) {
+                aperts->bottom_x[k] = i+1;
+                aperts->bottom_y[k] = j+1;
+            }
+            if (j+1 > aperts->top_y[k]) {
+                aperts->top_x[k] = i+1;
+                aperts->top_y[k] = j+1;
+            }
+        }
+    }
+
+    /* Centroiding */
+    for (j = 0; j < ny; j++) {
+        for (i = 0; i < nx; i++) {
+            k = plab[i+j*nx]-1;
+
+            if (k >= 0) { /* Ignore background */
+                /* Current pixel value */
+                const double pix = pd[i+j*nx];
+
+                if (aperts->min_val[k] <= 0) {
+                    /* Accumulate weighted positions */
+                    aperts->xcentroid[k] +=
+                        (double)(i+1)*(pix-aperts->min_val[k]+ep);
+                    aperts->ycentroid[k] += 
+                        (double)(j+1)*(pix-aperts->min_val[k]+ep);
+                } else {
+                    /* Accumulate weighted positions */
+                    aperts->xcentroid[k] += (double)(i+1) * pix;
+                    aperts->ycentroid[k] += (double)(j+1) * pix;
+                }
+            }
+        }
+    }
+
+    /* Allocate buffers for the aperture medians, this will typically
+       require much less memory than the pixel buffer and in the worst
+       case on the same order */
+    medianc = cpl_malloc((size_t)naperts * sizeof(cpl_size));
+
+    /* Compute average and std dev for each aperture, normalize centers */
+    for (k = 0; k < aperts->naperts; k++) {
+        double weight;
+        const cpl_size npix = aperts->npix[k];
+        if (npix <= 0) {
+            cpl_apertures_delete(aperts);
+            cpl_image_delete(cimage);
+            cpl_image_delete(labcopy);
+            cpl_free(medianc);
+            (void)cpl_error_set_message_(CPL_ERROR_DATA_NOT_FOUND, "Label image"
+                                         " of size %d X %d with %d label(s) "
+                                         "has no pixel labeled %d", (int)nx,
+                                         (int)ny, (int)naperts, (int)k);
+            return NULL;
+        }
+
+        /* Determine end index for each median */
+        medianc[k] = k > 0 ? npix + medianc[k - 1] : npix;
+
+        aperts->stdev[k] = npix > 1 ?
+            sqrt(aperts->varsum[k]/((double)npix-1.0)) : -1.0;
+
+        aperts->x[k] /= (double)npix;
+        aperts->y[k] /= (double)npix;
+        weight = aperts->flux[k];
+        if (aperts->min_val[k] <= 0) {
+            weight -= (double)npix * aperts->min_val[k];
+            weight += (double)npix * ep;
+        }
+
+        if (aperts->xcentroid[k] >= weight * (double)aperts->left_x[k] &&
+            aperts->xcentroid[k] <= weight * (double)aperts->right_x[k] &&
+            weight != 0.0) {
+            aperts->xcentroid[k] /= weight;
+        } else {
+            aperts->xcentroid[k] = 0.0;
+        }
+
+        if (aperts->ycentroid[k] >= weight * (double)aperts->bottom_y[k] &&
+            aperts->ycentroid[k] <= weight * (double)aperts->top_y[k] &&
+            weight != 0.0) {
+            aperts->ycentroid[k] /= weight;
+        } else {
+            aperts->ycentroid[k] = 0.0;
+        }
+    }
+
+    /* Allocate space for all pixels belonging to an aperture */
+    medians = cpl_malloc((size_t)medianc[aperts->naperts-1] * sizeof(*medians));
+
+    /* Repack all pixels belonging to an aperture
+       into an array specific to that aperture */
+    for (i = 0; i < nx * ny; i++) {
+        k = plab[i]-1;
+
+        if (k >= 0) { /* Ignore background */
+            /* Insert value into its array, while adjusting the index for the
+               next access */
+            medians[--medianc[k]] = pd[i];
+        }
+    }
+
+    /* Compute median for each aperture */
+    assert(medianc[0] == 0);
+    aperts->median[0] = cpl_tools_get_median_double(medians, aperts->npix[0]);
+    for (k = 1; k < aperts->naperts; k++) {
+
+        assert(medianc[k] == medianc[k - 1] + aperts->npix[k - 1]);
+
+        aperts->median[k] =
+            cpl_tools_get_median_double(medians + medianc[k], aperts->npix[k]);
+    }
+
+    cpl_free(medians);
+    cpl_free(medianc);
+    cpl_image_delete(cimage);
+    cpl_image_delete(labcopy);
+
+    return aperts;
+}
