@@ -45,6 +45,56 @@
  * @endcode
  */
 
+/* The "triangle" is actually the element 2D pattern made of 3 points.
+   It carries its coordinates in the parameter space of similar
+   triangles, and an identifier of the three points composing it.
+   The identifiers are the matrix column sequence number in the
+   matrix containing the points (in the main program). */
+
+typedef struct _triangle
+{
+    double ratsq;           /* (Rmin/Rmax)^2                      */
+    double dratsq;          /* Error                              */
+
+    double theta;           /* Angle min - Angle max  in [0; 2pi[ */
+    double dtheta;          /* Error                              */
+
+    double xref, yref;      /* Reference point                    */
+    double xmin, ymin;      /* Nearest point                      */
+    double xmax, ymax;      /* Farthest point                     */
+
+    int    id_ref;          /* Identifier of reference point      */  
+    int    id_min;          /* Identifier of nearest point        */  
+    int    id_max;          /* Identifier of farthest point       */  
+
+} triangle;
+
+/* Here we have a set of triangles... */
+
+typedef struct _triangles
+{
+    int        n;
+    triangle **t;
+} triangles;
+
+static cpl_array *
+find_all_matches(const cpl_matrix *, const cpl_matrix *, 
+                 double, const cpl_polynomial *, 
+                 const cpl_polynomial *,
+                 cpl_matrix **, cpl_matrix **) CPL_ATTR_ALLOC;
+
+static cpl_array *
+find_matches(const triangles *, const triangles *, int) CPL_ATTR_ALLOC;
+
+static triangles *triangles_new(int) CPL_ATTR_ALLOC;
+
+static triangles *
+triangles_from_points(const cpl_matrix *, int, double) CPL_ATTR_ALLOC;
+
+static triangle *
+triangle_new(double, double, double, double, double, double,
+             double, int, int, int) CPL_ATTR_ALLOC;
+
 /**@{*/
 
 /**
@@ -726,7 +776,7 @@ cpl_bivector *cpl_ppm_match_positions(const cpl_vector *peaks,
 static double 
 double_subtract(double x, double dx, double y, double dy, double *error)
 {
-    *error = sqrt( dx*dx + dy*dy );
+    *error = hypot(dx, dy);
     return x - y;
 }
 
@@ -735,7 +785,7 @@ double_subtract(double x, double dx, double y, double dy, double *error)
 static double 
 double_divide(double x, double dx, double y, double dy, double *error)
 {   
-    double y2 = y*y;
+    const double y2 = y*y;
 
 
     if (y2 > 0.0) {
@@ -753,8 +803,8 @@ double_divide(double x, double dx, double y, double dy, double *error)
 static double 
 double_atan2(double y, double dy, double x, double dx, double *error)
 {
-    double x2 = x*x;
-    double y2 = y*y;
+    const double x2 = x*x;
+    const double y2 = y*y;
 
 
     if (x2 > 0.0 || y2 > 0.0) {
@@ -780,34 +830,6 @@ point_distsq(double xa, double ya, double xb, double yb)
     return ((xa - xb)*(xa - xb) + (ya - yb)*(ya - yb));
 }
 
-/* The "triangle" is actually the element 2D pattern made of 3 points.
-   It carries its coordinates in the parameter space of similar
-   triangles, and an identifier of the three points composing it.
-   The identifiers are the matrix column sequence number in the
-   matrix containing the points (in the main program). */
-
-typedef struct _triangle
-{
-    double ratsq;           /* (Rmin/Rmax)^2                      */
-    double dratsq;          /* Error                              */
-
-    double theta;           /* Angle min - Angle max  in [0; 2pi[ */
-    double dtheta;          /* Error                              */
-
-    double xref, yref;      /* Reference point                    */
-    double xmin, ymin;      /* Nearest point                      */
-    double xmax, ymax;      /* Farthest point                     */
-
-    int    id_ref;          /* Identifier of reference point      */  
-    int    id_min;          /* Identifier of nearest point        */  
-    int    id_max;          /* Identifier of farthest point       */  
-
-} triangle;
-
-/* Square root of 2 */
-
-#define V2 (1.4142136)
-
 /* This constructs a triangle from three points, propagates errors,
    and keeps the points identifiers. */
 
@@ -815,24 +837,24 @@ static triangle *
 triangle_new(double xa, double ya, double xb, double yb, double xc, double yc,
              double error, int ida, int idb, int idc)
 {
-    double    r1, r2, dr1, dr2;
-    double    t1, t2, dt1, dt2;
-    double    two_pi;
-    triangle *t;
+    const double r1 = point_distsq(xa, ya, xb, yb);
+    const double r2 = point_distsq(xa, ya, xc, yc);
+    const double dr1 = sqrt(8*error*error*r1);
+    const double dr2 = sqrt(8*error*error*r2);
 
+    double  dt1, dt2;
+
+    const double t1 = double_atan2(ya - yb, CPL_MATH_SQRT2 * error,
+                                   xa - xb, CPL_MATH_SQRT2 * error, &dt1);
+    const double t2 = double_atan2(ya - yc, CPL_MATH_SQRT2 * error,
+                                   xa - xc, CPL_MATH_SQRT2 * error, &dt2);
+
+    triangle *t = cpl_calloc(1, sizeof(triangle));
 
-    t = cpl_calloc(1, sizeof(triangle));
     t->xref = xa;
     t->yref = ya;
     t->id_ref = ida;
 
-    r1 = point_distsq(xa, ya, xb, yb);
-    r2 = point_distsq(xa, ya, xc, yc);
-    dr1 = sqrt(8*error*error*r1);
-    dr2 = sqrt(8*error*error*r2);
-    t1 = double_atan2(ya - yb, V2*error, xa - xb, V2*error, &dt1);
-    t2 = double_atan2(ya - yc, V2*error, xa - xc, V2*error, &dt2);
-
     if (r1 < r2) {
         t->ratsq  = double_divide(r1, dr1, r2, dr2, &t->dratsq);
         t->theta  = double_subtract(t1, dt1, t2, dt2, &t->dtheta);
@@ -842,8 +864,7 @@ triangle_new(double xa, double ya, double xb, double yb, double xc, double yc,
         t->xmax   = xc;
         t->ymax   = yc;
         t->id_max = idc;
-    }
-    else {
+    } else {
         t->ratsq  = double_divide(r2, dr2, r1, dr1, &t->dratsq);
         t->theta  = double_subtract(t2, dt2, t1, dt1, &t->dtheta);
         t->xmin   = xc;
@@ -854,13 +875,11 @@ triangle_new(double xa, double ya, double xb, double yb, double xc, double yc,
         t->id_max = idb;
     }
 
-    two_pi = CPL_MATH_2PI;
+    while (t->theta < 0.0) 
+        t->theta += CPL_MATH_2PI;
 
-    while (t->theta < 0) 
-        t->theta += two_pi;
-
-    while (t->theta >= two_pi) 
-        t->theta -= two_pi;
+    while (t->theta >= CPL_MATH_2PI) 
+        t->theta -= CPL_MATH_2PI;
 
     return t;
 }
@@ -874,21 +893,12 @@ triangle_delete(triangle **t)
     *t = NULL;
 }
 
-/* Here we have a set of triangles... */
-
-typedef struct _triangles
-{
-    int        n;
-    triangle **t;
-} triangles;
-
 /* A constructor of arrays of triangles... */
 
 static triangles *triangles_new(int n)
 {
-    triangles *t;
+    triangles *t = cpl_calloc(1, sizeof(triangles));
 
-    t    = cpl_calloc(1, sizeof(triangles));
     t->t = cpl_calloc((size_t)n, sizeof(triangle *));
     t->n = n;
  
@@ -922,9 +932,8 @@ triangles_from_points(const cpl_matrix *points, int n, double err)
 {
     triangles     *pattern;
     const double  *m = cpl_matrix_get_data_const(points);
-    double         x1, x2, x3, y1, y2, y3;
     const cpl_size nc = cpl_matrix_get_ncol(points);
-    int            nt = n*(n-1)*(n-2)/2;
+    const int      nt = n*(n-1)*(n-2)/2;
     int            count;
     int            i, j, k;
 
@@ -937,6 +946,7 @@ triangles_from_points(const cpl_matrix *points, int n, double err)
     for (i = 0; i < n-2; i++) {
         for (j = i+1; j < n-1; j++) {
             for (k = j+1; k < n; k++) {
+                double         x1, x2, x3, y1, y2, y3;
 
                 /*
                  * i, j, and k, are the matrix columns where the
@@ -974,8 +984,18 @@ triangles_from_points(const cpl_matrix *points, int n, double err)
 static double 
 difference_of_angles(double a1, double a2)
 {
-    double d = a1 - a2;
 
+    double d;
+
+    /* NB: Handle a1 = 2pi - epsilon, a2 = epsilon => d = 2 epsilon */
+    while (a1 < -CPL_MATH_PI) a1 += CPL_MATH_2PI;
+    while (a1 >  CPL_MATH_PI) a1 -= CPL_MATH_2PI;
+    while (a2 < -CPL_MATH_PI) a2 += CPL_MATH_2PI;
+    while (a2 >  CPL_MATH_PI) a2 -= CPL_MATH_2PI;
+
+    d = a1 - a2;
+
+   /* NB: Handle a1 = pi - epsilon, a2 = epsilon - pi => d = 2 epsilon */
     while (d < -CPL_MATH_PI) d += CPL_MATH_2PI;
     while (d >  CPL_MATH_PI) d -= CPL_MATH_2PI;
 
@@ -989,26 +1009,24 @@ difference_of_angles(double a1, double a2)
  */
 
 static double
-distance_of_triangles(triangle *t1, triangle *t2, double *error)
+distance_of_triangles(const triangle *t1, const triangle *t2, double *error)
 {
-    double dtheta = difference_of_angles(t1->theta, t2->theta); /* in [0;pi] */
-    double dist, dist_in_errs, dr, pisquare, rr, tt;
+    const double dtheta =
+        difference_of_angles(t1->theta, t2->theta); /* in [0;pi] */
+    const double dr = t1->ratsq - t2->ratsq;
+    const double rr = t1->dratsq*t1->dratsq + t2->dratsq*t2->dratsq;
+    const double tt = (t1->dtheta*t1->dtheta + t2->dtheta*t2->dtheta) / 4;
 
-
-    dtheta *= dtheta;
-    dr = (t1->ratsq - t2->ratsq) * (t1->ratsq - t2->ratsq);
-    pisquare = CPL_MATH_PI*CPL_MATH_PI;
-    rr = t1->dratsq*t1->dratsq + t2->dratsq*t2->dratsq;
-    tt = (t1->dtheta*t1->dtheta + t2->dtheta*t2->dtheta) / 4;
-
-    dist = sqrt(dr + dtheta / pisquare);
+    const double dist = hypot(dr, dtheta / CPL_MATH_PI);
 
     if (error) {
-        dist_in_errs = sqrt(dr/rr + dtheta/tt);
-        if (dist == 0.0)
-            *error = 1.0;
-        else
+        if (dist != 0.0) {
+            const double dist_in_errs =
+                sqrt((dr * dr)/rr + (dtheta * dtheta)/tt);
             *error = dist / dist_in_errs;
+        } else {
+            *error = 1.0;
+        }
     }
 
     return dist;
@@ -1018,7 +1036,7 @@ static triangle *
 nearest_triangle(triangle *t, triangles *list)
 {
     triangle **pattern = list->t;
-    int        n       = list->n;
+    const int  n       = list->n;
     triangle  *nearest;
 
     double     dist, mindist;
@@ -1046,10 +1064,10 @@ nearest_triangle(triangle *t, triangles *list)
 static double 
 scaling_factor_of_triangles(const triangle *t1, const triangle *t2)
 {
-    double s1 = point_distsq(t1->xref, t1->yref, t1->xmax, t1->ymax);
-    double s2 = point_distsq(t2->xref, t2->yref, t2->xmax, t2->ymax);
+    const double s1 = point_distsq(t1->xref, t1->yref, t1->xmax, t1->ymax);
+    const double s2 = point_distsq(t2->xref, t2->yref, t2->xmax, t2->ymax);
 
-    return (s2 == 0) ? 0 : sqrt(s1/s2);
+    return (s2 != 0.0) ? sqrt(s1/s2) : 0.0;
 }
 
 
@@ -1060,8 +1078,8 @@ scaling_factor_of_triangles(const triangle *t1, const triangle *t2)
 static double 
 angle_between_triangles(const triangle *t1, const triangle *t2)
 {
-    double a1 = atan2(t1->yref - t1->ymax, t1->xref - t1->xmax);
-    double a2 = atan2(t2->yref - t2->ymax, t2->xref - t2->xmax);
+    const double a1 = atan2(t1->yref - t1->ymax, t1->xref - t1->xmax);
+    const double a2 = atan2(t2->yref - t2->ymax, t2->xref - t2->xmax);
     double a = a1 - a2;
 
     while (a >= CPL_MATH_2PI) 
@@ -1074,7 +1092,7 @@ angle_between_triangles(const triangle *t1, const triangle *t2)
 
 
 static cpl_array *
-find_matches(triangles *p_triangle, triangles *n_triangle, int np)
+find_matches(const triangles *p_triangle, const triangles *n_triangle, int np)
 {
     int        nt = p_triangle->n;
     int        i;
@@ -1104,7 +1122,7 @@ find_matches(triangles *p_triangle, triangles *n_triangle, int np)
 /* Returns 0 on success */
 
 static int
-find_transform(cpl_array *matches, const cpl_matrix *pattern, 
+find_transform(const cpl_array *matches, const cpl_matrix *pattern, 
                const cpl_matrix *data, cpl_size degree, 
                cpl_polynomial *trans_x, cpl_polynomial *trans_y) 
 {
@@ -1533,7 +1551,6 @@ cpl_ppm_match_points(const cpl_matrix *data, cpl_size use_data, double err_data,
     triangles      *d_triangle;   /* Triangles from data points             */
     triangles      *p_triangle;   /* Triangles from pattern points          */
     triangles      *n_triangle;   /* Nearest d_triangles to each p_triangle */
-    triangle       *nearest;      /* Nearest triangle                       */
     cpl_array      *matches = NULL;
     cpl_table      *table = NULL;
     cpl_polynomial *trans_x;
@@ -1670,7 +1687,8 @@ cpl_ppm_match_points(const cpl_matrix *data, cpl_size use_data, double err_data,
      */
 
     for (i = 0; i < p_triangle->n; i++) {
-        nearest = nearest_triangle(p_triangle->t[i], d_triangle);
+        triangle  *nearest =   /* Nearest triangle */
+            nearest_triangle(p_triangle->t[i], d_triangle);
         distance[i] = distance_of_triangles(p_triangle->t[i], nearest, &err);
         if (distance[i] / err < 1.0) {
             scale[i] = scaling_factor_of_triangles(nearest, p_triangle->t[i]);
@@ -1707,12 +1725,16 @@ cpl_ppm_match_points(const cpl_matrix *data, cpl_size use_data, double err_data,
 #ifdef CPL_PPM_DEBUG
         const double rms_s    = cpl_table_get_column_stdev(table, "angle_s");
         const double rms_c    = cpl_table_get_column_stdev(table, "angle_c");
-        const double rms_angle= sqrt(rms_s*rms_s + rms_c*rms_c);
+        const double rms_angle= hypot(rms_s, rms_c);
         double rms_scale;
 #endif
         const double median_c = cpl_table_get_column_median(table, "angle_c");
 
         median_scale = cpl_table_get_column_median(table, "scale");
+
+        /* The circular (i.e. non-Euclidian) nature of the angle makes it
+           unsuitable for the median (and mean), so instead take the median
+           of the Cartesian coordinates */
         median_angle = atan2(median_s, median_c);
 
 #ifdef CPL_PPM_DEBUG
@@ -1727,7 +1749,8 @@ cpl_ppm_match_points(const cpl_matrix *data, cpl_size use_data, double err_data,
 #endif
     }
     else {
-        cpl_msg_debug(cpl_func, "No match found");
+        cpl_msg_debug(cpl_func, "No match found among %d = %d row(s)",
+                      (int)cpl_table_get_nrow(table), (int)(p_triangle->n));
         cpl_table_delete(table);
         triangles_delete(&p_triangle);
         triangles_delete(&d_triangle);
@@ -1752,6 +1775,12 @@ cpl_ppm_match_points(const cpl_matrix *data, cpl_size use_data, double err_data,
         if (n_triangle->t[i]) {
             if (fabs(scale[i] - median_scale) > scale_tolerance
             || difference_of_angles(angle[i], median_angle) > angle_tolerance) {
+#ifdef CPL_PPM_DEBUG
+                cpl_msg_debug(cpl_func, "Discarding %d/%d: |%g - %g| > %g * %g or "
+                              "diff_angle(%g, %g) > %g * |%g|", i+1, p_triangle->n,
+                              scale[i], median_scale, tolerance, median_scale,
+                              angle[i], median_angle, tolerance, median_angle);
+#endif
                 n_triangle->t[i] = NULL;
                 cpl_table_set_invalid(table, "distance", i);
                 cpl_table_set_invalid(table, "scale", i);
@@ -1772,7 +1801,7 @@ cpl_ppm_match_points(const cpl_matrix *data, cpl_size use_data, double err_data,
 #ifdef CPL_PPM_DEBUG
         const double rms_s    = cpl_table_get_column_stdev(table, "angle_s");
         const double rms_c    = cpl_table_get_column_stdev(table, "angle_c");
-        const double rms_angle= sqrt(rms_s*rms_s + rms_c*rms_c);
+        const double rms_angle= hypot(rms_s, rms_c);
         double rms_scale;
 #endif
         const double median_c = cpl_table_get_column_median(table, "angle_c");
@@ -1797,7 +1826,8 @@ cpl_ppm_match_points(const cpl_matrix *data, cpl_size use_data, double err_data,
         cpl_error_set_message(cpl_func, CPL_ERROR_UNSPECIFIED, 
                               "No match found (impossible!)");
 */
-        cpl_msg_debug(cpl_func, "No match found");
+        cpl_msg_debug(cpl_func, "No match found among %d row(s)",
+                      (int)cpl_table_get_nrow(table));
         cpl_table_delete(table);
         triangles_delete(&p_triangle);
         triangles_delete(&d_triangle);
@@ -1871,6 +1901,4 @@ cpl_ppm_match_points(const cpl_matrix *data, cpl_size use_data, double err_data,
    
 }
 
-#undef V2
-
 /**@}*/
