@@ -26,22 +26,21 @@
                                    Includes
  ----------------------------------------------------------------------------*/
 
-
-#include <math.h>
-#include <stdio.h>
-#include <string.h>
-#include <fitsio.h>
-
 #include <cpl_memory.h>
 #include <cpl_tools.h>
 #include <cpl_error_impl.h>
 #include <cpl_propertylist_impl.h>
+#include <cpl_property_impl.h>
 #include <cpl_math_const.h>
 #include <cpl_errorstate.h>
 
 #include "cpl_wcs.h"
 
-
+#include <math.h>
+#include <stdio.h>
+#include <string.h>
+#include <fitsio.h>
+#include <assert.h>
 
 #ifdef CPL_WCS_INSTALLED    /* If WCS is installed */
 
@@ -930,44 +929,47 @@ cpl_error_code cpl_wcs_platesol(const cpl_propertylist *ilist,
     } else {
         cpl_errorstate    prestate = cpl_errorstate_get();
         cpl_propertylist *pvlist;
-        cpl_property     *p;
+        cpl_property     *p = NULL;
         char             *outstr = cpl_calloc((size_t)(i+1) * 80 + 1, 1);
         /* FITS END card */
         const char *endcard = "END                            "
             "                                                 ";
         cpl_size nprop, iprop;
+        char *cd    = NULL;
+        int   cdlen = 0;
 
         strncpy(outstr, o, (size_t)i * 80);
         free(o);
-        strncat(outstr,endcard,80);
+        strncat(outstr,endcard,79);
         *olist = cpl_wcs_fitsstr2plist(outstr);
         cpl_free(outstr);
 
 
         for (i = 2; i > 0; --i) {
-
-            unsigned int j;
-
-            for (j = 2; j > 0; --j)
+            for (int j = 2; j > 0; j--)
             {
-                char *cd = cpl_sprintf("CD%1d_%1d", i, j);
-                cpl_property *_p = cpl_property_new(cd,
-                                                    CPL_TYPE_DOUBLE);
+                const double cd_val = cpl_matrix_get(wcs->cd, i - 1, j - 1);
 
-                cpl_property_set_double(_p,
-                                        cpl_matrix_get(wcs->cd, i - 1, j - 1));
-                cpl_property_set_comment(_p, "Coordinate transformation matrix "
-                                         "element");
+                if (p == NULL) {
+                    cd = cpl_sprintf("CD%1d_%1d%n", i, j, &cdlen);
+                    p = cpl_property_new_cx(CXSTR(cd, cdlen), CPL_TYPE_DOUBLE);
+                    cpl_property_set_comment_cx(p, CXSTR("Coordinate "
+                                                         "transformation "
+                                                         "matrix element",
+                                                         40));
+                } else {
+                    const int keylen = sprintf(cd, "CD%1d_%1d", i, j);
+                    assert(keylen == cdlen);
+                    cpl_property_set_name_cx(p, CXSTR(cd, cdlen));
+                }
 
-                cpl_free(cd);
-
-                cpl_propertylist_insert_after_property(*olist, "CRPIX2", _p);
-                cpl_property_delete(_p);
+                cpl_property_set_double(p, cd_val);
 
+                cpl_propertylist_insert_after_property(*olist, "CRPIX2", p);
             }
-
         }
-
+        cpl_property_delete(p);
+        cpl_free(cd);
 
         /*
          * Remove keywords which are not needed for the DICB required
@@ -1221,7 +1223,6 @@ const cpl_array *cpl_wcs_get_crval(const cpl_wcs *wcs) {
     return wcs->wcsptr->naxis == 0 ? NULL : wcs->crval;
 
 #else
-    cpl_ensure(wcs != NULL, CPL_ERROR_NULL_INPUT, NULL);
     (void)cpl_error_set_(CPL_ERROR_NO_WCS);
     return NULL;
 #endif                      /* End If WCS is installed */
@@ -1267,7 +1268,6 @@ const cpl_array *cpl_wcs_get_crpix(const cpl_wcs *wcs) {
     return wcs->wcsptr->naxis == 0 ? NULL : wcs->crpix;
 
 #else
-    cpl_ensure(wcs != NULL, CPL_ERROR_NULL_INPUT, NULL);
     (void)cpl_error_set_(CPL_ERROR_NO_WCS);
     return NULL;
 #endif                      /* End If WCS is installed */
@@ -1313,7 +1313,6 @@ const cpl_array *cpl_wcs_get_ctype(const cpl_wcs *wcs) {
     return wcs->wcsptr->naxis == 0 ? NULL : wcs->ctype;
 
 #else
-    cpl_ensure(wcs != NULL, CPL_ERROR_NULL_INPUT, NULL);
     (void)cpl_error_set_(CPL_ERROR_NO_WCS);
     return NULL;
 #endif                      /* End If WCS is installed */
@@ -1359,7 +1358,6 @@ const cpl_array *cpl_wcs_get_cunit(const cpl_wcs *wcs) {
     return wcs->wcsptr->naxis == 0 ? NULL : wcs->cunit;
 
 #else
-    cpl_ensure(wcs != NULL, CPL_ERROR_NULL_INPUT, NULL);
     (void)cpl_error_set_(CPL_ERROR_NO_WCS);
     return NULL;
 #endif                      /* End If WCS is installed */
