@@ -1,7 +1,6 @@
-/* $Id: cpl_wlcalib.c,v 1.35 2013-03-25 10:20:40 cgarcia Exp $
- *
- * This file is part of the CPL Pipeline
- * Copyright (C) 2002,2003 European Southern Observatory
+/*
+ * This file is part of the ESO Common Pipeline Library
+ * Copyright (C) 2001-2017 European Southern Observatory
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,13 +17,6 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
-/*
- * $Author: cgarcia $
- * $Date: 2013-03-25 10:20:40 $
- * $Revision: 1.35 $
- * $Name: not supported by cvs2svn $
- */
-
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -817,8 +809,13 @@ cpl_wlcalib_fill_logline_spectrum_fast(cpl_vector            * self,
  Find the polynomial that maximizes the cross-correlation between an observed
  1D-spectrum and a model spectrum based on the polynomial dispersion relation. 
 
- The wavelength search range is in the same units as the Y-values of the
- dispersion relation.
+ Each element in the vector of wavelength search ranges is in the same unit as
+ the corresponding Y-value of the dispersion relation. Each value in the vector
+ is the width of a search window centered on the corresponding value in the
+ guess polynomial. The length D of the search vector thus determines the
+ dimensionality of the search space for the dispersion polynomial. If for
+ example the search vector consists of three elements, then the three lowest
+ order coefficients of the dispersion relation may be modified by the search.
 
  For each candidate polynomial P(x), the polynomial P(x+u), -hsize <= u <= hsize
  is also evaluated. The half-size hsize may be zero. When it is non-zero, an
@@ -830,7 +827,7 @@ cpl_wlcalib_fill_logline_spectrum_fast(cpl_vector            * self,
 
  The complexity in terms of model spectra creation is O(N^D) and in terms of
  cross-correlations O(hsize * N^D), where N is nsamples and D is the length
- of wl_error.
+ of wl_search.
 
  xcorrs must be NULL or have a size of (at least) N^D*(1 + 2 * hsize).
  
