@@ -55,6 +55,8 @@
 
 #define IMSZ 6
 
+#define IM13 13
+
 /*----------------------------------------------------------------------------*/
 /**
  * @defgroup cpl_fit_test Testing of the CPL utilities
@@ -77,19 +79,22 @@
 
 static void cpl_fit_imagelist_polynomial_tests(void);
 static void cpl_fit_imagelist_polynomial_bpm(void);
-static void cpl_fit_image_gaussian_tests(cpl_type);
+static void cpl_fit_image_gaussian_tests(cpl_type, FILE *);
 static void cpl_fit_imagelist_is_zero_macro(const cpl_imagelist *, double,
                                                int);
 static void cpl_fit_image_is_zero_macro(const cpl_image *, double, int);
 static void eval_gauss(const double x[], const double a[], double *);
 static int  check_gauss_success(const char **, const double *, const double *,
                                 const char **, const double *, const double *,
-                                cpl_array   *, cpl_array    *, 
-                                double, double, double, cpl_matrix   *);
+                                const cpl_array *, const cpl_array *, 
+                                double, double, double, const cpl_matrix *,
+                                FILE *);
 static void cpl_fit_image_gaussian_test_one(void);
 
 static void cpl_fit_image_gaussian_test_local(void);
 
+static void cpl_fit_image_gaussian_test_bpm(void);
+
 static
 cpl_imagelist * cpl_fit_imagelist_polynomial_window_test(const cpl_vector    *,
                                                          const cpl_imagelist *,
@@ -106,17 +111,26 @@ cpl_imagelist * cpl_fit_imagelist_polynomial_window_test(const cpl_vector    *,
 
 int main(void)
 {
+    FILE        * stream;
 
     cpl_test_init(PACKAGE_BUGREPORT, CPL_MSG_WARNING);
 
+    stream = cpl_msg_get_level() > CPL_MSG_INFO
+        ? fopen("/dev/null", "a") : stdout;
+
+    cpl_test_nonnull( stream );
+
     /* Insert tests below */
     cpl_fit_imagelist_polynomial_bpm();
-    cpl_fit_image_gaussian_tests(CPL_TYPE_INT);
-    cpl_fit_image_gaussian_tests(CPL_TYPE_FLOAT);
-    cpl_fit_image_gaussian_tests(CPL_TYPE_DOUBLE);
+    cpl_fit_image_gaussian_tests(CPL_TYPE_INT, stream);
+    cpl_fit_image_gaussian_tests(CPL_TYPE_FLOAT, stream);
+    cpl_fit_image_gaussian_tests(CPL_TYPE_DOUBLE, stream);
     cpl_fit_imagelist_polynomial_tests();
     cpl_fit_image_gaussian_test_one();
     cpl_fit_image_gaussian_test_local();
+    cpl_fit_image_gaussian_test_bpm();
+
+    if (stream != stdout) cpl_test_zero( fclose(stream) );
 
     /* End of tests */
     return cpl_test_end(0);
@@ -143,8 +157,10 @@ static void cpl_fit_imagelist_polynomial_tests(void)
         = cpl_image_new(IMAGESZFIT, IMAGESZFIT, CPL_TYPE_FLOAT_COMPLEX);
     cpl_image      * dfiterrorwin;
     const int        ndits = (int)(sizeof(ditval)/sizeof(double));
+    CPL_DIAG_PRAGMA_PUSH_IGN(-Wcast-qual)
     cpl_vector     * vdit  = cpl_vector_wrap(ndits, (double*)ditval);
     cpl_vector     * nvdit = cpl_vector_wrap(ndits, (double*)neqditval);
+    CPL_DIAG_PRAGMA_POP
     cpl_polynomial * vfiller = cpl_polynomial_new(1);
     const double     sqsum = 204.0; /* Sum of squares of ditvals */
     const double     mytol = 5.52 * FLT_EPSILON;
@@ -962,7 +978,7 @@ static void cpl_fit_image_is_zero_macro(const cpl_image * self, double tol,
 
 }
 
-static void cpl_fit_image_gaussian_tests(cpl_type pixeltype)
+static void cpl_fit_image_gaussian_tests(cpl_type pixeltype, FILE * stream)
 {
     cpl_image  *box        = NULL;
     cpl_image  *image      = NULL;
@@ -1080,20 +1096,28 @@ static void cpl_fit_image_gaussian_tests(cpl_type pixeltype)
      * Now see if we can find a gaussian without any first guess:
      */
 
-    phys_cov = NULL;
-    covariance = NULL;
-    error = cpl_fit_image_gaussian(image, eimage, 797, 1976, 30, 30,
-                                   parameters, err_params, fit_params, 
-                                   &rms, &chisq, &covariance, &major, 
-                                   &minor, &angle, &phys_cov);
-    cpl_test_eq_error(error, CPL_ERROR_NONE);
-    if (!error || cpl_msg_get_level() <= CPL_MSG_INFO)
-        cpl_test(check_gauss_success(p, u, e, fp, fu, fe, parameters,
-                                     err_params, angle, major, minor,
-                                     phys_cov));
+    for (int itry = 0; itry < 2; itry++) {
+
+        phys_cov = NULL;
+        covariance = NULL;
+        error = cpl_fit_image_gaussian(image, eimage, 797, 1976, 30, 30,
+                                       parameters, err_params, fit_params, 
+                                       &rms, &chisq, &covariance, &major, 
+                                       &minor, &angle, &phys_cov);
+        cpl_test_eq_error(error, CPL_ERROR_NONE);
+        if (!error || cpl_msg_get_level() <= CPL_MSG_INFO)
+            cpl_test(check_gauss_success(p, u, e, fp, fu, fe, parameters,
+                                         err_params, angle, major, minor,
+                                         phys_cov, stream));
+
+        cpl_matrix_delete(covariance);
+        cpl_matrix_delete(phys_cov);
+
+        /*
+         * Now retry with the fit as first-guess, all parameters free.
+         */
+    }
 
-    cpl_matrix_delete(covariance);
-    cpl_matrix_delete(phys_cov);
 
     /*
      * Now retry with a first-guess, all parameters free.
@@ -1110,68 +1134,71 @@ static void cpl_fit_image_gaussian_tests(cpl_type pixeltype)
     for (i = 0; i < 7; i++)
         cpl_array_set(err_params, i, e[i]);
 
-    phys_cov = NULL;
-    covariance = NULL;
-    error = cpl_fit_image_gaussian(image, eimage, 797, 1976, 30, 30,
-                                   parameters, err_params, fit_params, 
-                                   &rms, &chisq, &covariance, &major, 
-                                   &minor, &angle, &phys_cov);
-    cpl_test_eq_error(error, CPL_ERROR_NONE);
+    for (int itry = 0; itry < 2; itry++) {
 
-    if (!error || cpl_msg_get_level() <= CPL_MSG_INFO)
-        cpl_test(check_gauss_success(p, u, e, fp, fu, fe, parameters,
-                                     err_params, angle, major, minor,
-                                     phys_cov));
+        phys_cov = NULL;
+        covariance = NULL;
+        error = cpl_fit_image_gaussian(image, eimage, 797, 1976, 30, 30,
+                                       parameters, err_params, fit_params, 
+                                       &rms, &chisq, &covariance, &major, 
+                                       &minor, &angle, &phys_cov);
+        cpl_test_eq_error(error, CPL_ERROR_NONE);
 
-    cpl_matrix_delete(covariance);
-    cpl_matrix_delete(phys_cov);
+        if (!error || cpl_msg_get_level() <= CPL_MSG_INFO)
+            cpl_test(check_gauss_success(p, u, e, fp, fu, fe, parameters,
+                                         err_params, angle, major, minor,
+                                         phys_cov, stream));
 
-    /*
-     * Now retry with a first-guess, two parameters are frozen.
-     */
+        cpl_matrix_delete(covariance);
+        cpl_matrix_delete(phys_cov);
 
-    for (i = 0; i < 7; i++)
-        cpl_array_set(parameters, i, u[i]);
+        /*
+         * Now retry with a first-guess, two parameters are frozen.
+         */
 
-    for (i = 0; i < 7; i++)
-        cpl_array_set(err_params, i, e[i]);
+        for (i = 0; i < 7; i++)
+            cpl_array_set(parameters, i, u[i]);
 
-    cpl_array_set(fit_params, 0, 0);
-    cpl_array_set(fit_params, 4, 0);
+        for (i = 0; i < 7; i++)
+            cpl_array_set(err_params, i, e[i]);
 
-    phys_cov = NULL;
-    covariance = NULL;
-    error = cpl_fit_image_gaussian(image, eimage, 797, 1976, 30, 30,
-                                   parameters, err_params, fit_params,
-                                   &rms, &chisq, &covariance, &major,
-                                   &minor, &angle, &phys_cov);
-    cpl_test_eq_error(error, CPL_ERROR_NONE);
+        cpl_array_set(fit_params, 0, 0);
+        cpl_array_set(fit_params, 4, 0);
 
-    if (!error || cpl_msg_get_level() <= CPL_MSG_INFO)
-        cpl_test(check_gauss_success(p, u, e, fp, fu, fe, parameters,
-                                     err_params, angle, major, minor,
-                                     phys_cov));
+        phys_cov = NULL;
+        covariance = NULL;
+        error = cpl_fit_image_gaussian(image, eimage, 797, 1976, 30, 30,
+                                       parameters, err_params, fit_params,
+                                       &rms, &chisq, &covariance, &major,
+                                       &minor, &angle, &phys_cov);
+        cpl_test_eq_error(error, CPL_ERROR_NONE);
 
-    cpl_matrix_delete(covariance);
-    cpl_matrix_delete(phys_cov);
+        if (!error || cpl_msg_get_level() <= CPL_MSG_INFO)
+            cpl_test(check_gauss_success(p, u, e, fp, fu, fe, parameters,
+                                         err_params, angle, major, minor,
+                                         phys_cov, stream));
+
+        cpl_matrix_delete(covariance);
+        cpl_matrix_delete(phys_cov);
 
 
-    /*
-     * Now try without eny error propagation
-     */
+        /*
+         * Now try without eny error propagation
+         */
 
-    phys_cov = NULL;
-    covariance = NULL;
-    error = cpl_fit_image_gaussian(image, NULL, 797, 1976, 30, 30,
-                                   parameters, NULL, fit_params,
-                                   &rms, NULL, NULL, &major,
-                                   &minor, &angle, NULL);
-    cpl_test_eq_error(error, CPL_ERROR_NONE);
+        phys_cov = NULL;
+        covariance = NULL;
+        error = cpl_fit_image_gaussian(image, NULL, 797, 1976, 30, 30,
+                                       parameters, NULL, fit_params,
+                                       &rms, NULL, NULL, &major,
+                                       &minor, &angle, NULL);
+        cpl_test_eq_error(error, CPL_ERROR_NONE);
 
-    if (!error || cpl_msg_get_level() <= CPL_MSG_INFO)
-        cpl_test(check_gauss_success(p, u, e, fp, fu, fe, parameters,
-                                     err_params, angle, major, minor,
-                                     phys_cov));
+        if (!error || cpl_msg_get_level() <= CPL_MSG_INFO)
+            cpl_test(check_gauss_success(p, u, e, fp, fu, fe, parameters,
+                                         err_params, angle, major, minor,
+                                         phys_cov, stream));
+    }
 
     /*
      * Try different error situations:
@@ -1361,12 +1388,13 @@ static int check_gauss_success(const char  **p,
                                const char  **fp, 
                                const double *fu, 
                                const double *fe,
-                               cpl_array    *parameters,
-                               cpl_array    *err_params,
+                               const cpl_array *parameters,
+                               const cpl_array *err_params,
                                double        angle,
                                double        major,
                                double        minor,
-                               cpl_matrix   *phys_cov)
+                               const cpl_matrix *phys_cov,
+                               FILE * stream)
 {
     cpl_matrix *matrix  = NULL;
     int         success = 0;
@@ -1390,6 +1418,8 @@ static int check_gauss_success(const char  **p,
         error = cpl_matrix_decomp_chol(matrix);
         cpl_test_eq_error(error, CPL_ERROR_NONE);
 
+        cpl_matrix_dump(phys_cov, stream);
+
     }
 
     for (i = 0; i < 7; i++) {
@@ -1641,7 +1671,9 @@ static void cpl_fit_imagelist_polynomial_bpm(void)
     const double     ditval[]    = {0.0, 1.0, 2.0, 3.0, 4.0, 
                                     5.0, 6.0, 7.0, 8.0};
     const int        ndits = (int)(sizeof(ditval)/sizeof(double));
+    CPL_DIAG_PRAGMA_PUSH_IGN(-Wcast-qual)
     cpl_vector     * vdit  = cpl_vector_wrap(ndits, (double*)ditval);
+    CPL_DIAG_PRAGMA_POP
 
     const int mx = 2; /* Two pixels: one all good, one mixed */
     const int my = 1;
@@ -1747,6 +1779,168 @@ static void cpl_fit_imagelist_polynomial_bpm(void)
     cpl_image_delete(dfiterror);
 }
 
+/*----------------------------------------------------------------------------*/
+/**
+  @internal
+  @brief   Test cpl_fit_image_gaussian() using local data
+  @return  void
+  @see PIPE-8125
+
+ */
+/*----------------------------------------------------------------------------*/
+static void cpl_fit_image_gaussian_test_bpm(void)
+{
+    cpl_array * params = cpl_array_new(7, CPL_TYPE_DOUBLE);
+    const double values[IM13 * IM13]
+        = {856.781, 832.831, 876.031, 894.781, 1015.83, 1160.33, 1276.03, 1295.73, 1274.58, 1173.18, 1042.88, 970.481, 912.681,
+           894.575, 919.775, 925.975, 984.075, 1109.33, 1233.38, 1291.92, 1312.77, 1198.62, 1120.77, 1038.83, 1016.58, 975.075,
+           980.519, 980.419, 999.319, 1021.72, 1255.82, 1536.47, 1735.57, 1686.22, 1493.17, 1196.32, 1071.27, 1110.17, 1076.17,
+           1076.92, 1076.47, 1052.22, 1192.12, 1647.82, 2262.62, 2979.22, 3011.62, 2484.32, 1769.47, 1389.27, 1211.72, 1159.97,
+           1164.42, 1188.47, 1198.07, 1537.72, 2522.32, 4171.52, 5487.42, 5950.92, 4712.87, 3176.17, 1998.07, 1494.47, 1321.77,
+           1320.96, 1352.11, 1480.96, 2152.26, 3932.41, 6727.36, 9276.11, 9706.51, 7825.66, 5090.06, 2894.41, 1878.91, 1498.41,
+           1411.61, 1496.16, 1751.36, 2709.61, 5201.41, 8742.31, 12128.6, 12650.6, -4.0375, 6567.81, 3663.51, 2145.31, 1623.26,
+           1411.12, 1544.52, 1822.82, 2780.97, 5355.67, 8861.92, 12061, 12846.3, 10185.7, 6598.87, 3677.82, 2073.72, 1517.12,
+           1377.82, 1478.97, 1764.62, 2456.02, 4256.12, 7063.37, 9463.02, 9937.97, 8086.52, 4928.32, 2861.67, 1698.72, 1353.57,
+           1219.54, 1313.94, 1447.39, 1844.84, 2740.39, 4471.04, 5562.24, 6004.69, 4866.64, 3301.69, 1990.74, 1307.34, 1171.99,
+           1057.14, 1130.09, 1146.79, 1242.09, 1617.19, 2364.34, 2982.64, 3218.14, 2812.74, 2019.04, 1492.14, 1201.34, 1051.94,
+           910.269, 929.319, 988.369, 1011.37, 1107.87, 1442.02, 1782.92, 1927.97, 1893.52, 1681.77, 1432.52, 1192.67, 1014.22,
+           889.488, 846.738, 876.587, 917.688, 802.588, 1168.19, 1421.54, 1701.69, 1707.54, 1562.19, 1312.04, 1156.39, 999.287};
+
+    const size_t image_bytes = IM13 * IM13 * sizeof(double);
+    double     * image_data  = (double*)cpl_malloc(image_bytes);
+    cpl_image  * image       = cpl_image_wrap_double(IM13, IM13, image_data);
+    cpl_image  * errim;
+
+    const double xfrac = 0.115;
+    const double yfrac = 0.511;
+
+    cpl_test_nonnull(image);
+
+    cpl_test_eq_ptr(memcpy(image_data, values, image_bytes), image_data);
+
+    errim = cpl_image_abs_create(image);
+    cpl_image_power(errim, 0.5);
+    cpl_image_reject_value(errim, CPL_VALUE_ZERO);
+
+    cpl_array_set(params, 3, 1.0 + (IM13 - 1) * 0.5 + xfrac);
+    cpl_array_set(params, 4, 1.0 + (IM13 - 1) * 0.5 + yfrac);
+
+    for (int idoerrim = 0; idoerrim < 2; idoerrim++) {
+        const cpl_image * use_errim = idoerrim ? errim : NULL;
+
+        cpl_msg_info(cpl_func, idoerrim ? "Testing with error image" :
+                     "Testing without error image");
+
+        for (int idobpm = 0; idobpm < 2; idobpm++) {
+            double rms0, major0, minor0, angle0;
+            double rms1 = 0.0, major1 = 0.0, minor1 = 0.0, angle1 = 0.0;
+            double drms = 0.0, dmajor = 0.0, dminor = 0.0, dangle = 0.0;
+            cpl_error_code code;
+
+            cpl_msg_info(cpl_func, idobpm ? "Testing with bpm" :
+                         "Testing without bpm");
+
+            if (idobpm) {
+                cpl_size nbad;
+                cpl_mask * nullmask;
+                cpl_mask * bpm = cpl_mask_threshold_image_create(image,
+                                                                 -FLT_MAX, 0.0);
+
+                cpl_test_error(CPL_ERROR_NONE);
+
+                nullmask = cpl_image_set_bpm(image, bpm);
+                cpl_test_error(CPL_ERROR_NONE);
+
+                cpl_test_null(nullmask);
+
+                nbad = cpl_image_count_rejected(image);
+                cpl_test_error(CPL_ERROR_NONE);
+                cpl_test_eq(nbad, 1);
+            }
+
+            for (int iretry = 0; iretry < 2; iretry++) {
+                cpl_matrix    * phys_cov  = NULL;
+                cpl_matrix    **pphys_cov = use_errim ? &phys_cov : NULL;
+
+                cpl_msg_info(cpl_func, iretry ? "Testing with first guess" :
+                             "Testing without first guess");
+
+                rms0 = rms1;
+                major0 = major1;
+                minor0 = minor1;
+                angle0 = angle1;
+
+                code = cpl_fit_image_gaussian(image, use_errim, IM13/2, IM13/2,
+                                              IM13, IM13, params, NULL, NULL,
+                                              &rms1, NULL, NULL, &major1,
+                                              &minor1, &angle1, pphys_cov);
+
+                if (code == CPL_ERROR_SINGULAR_MATRIX) {
+                    cpl_test(idoerrim);
+                    cpl_test_eq_error(code, CPL_ERROR_SINGULAR_MATRIX);
+                    cpl_test_null(phys_cov);
+
+                    /* Retry without covariance matrix */
+                    code = cpl_array_fill_window_invalid(params, 0, 7);
+                    cpl_test_eq_error(code, CPL_ERROR_NONE);
+                    cpl_array_set(params, 3, 1.0 + (IM13 - 1) * 0.5 + xfrac);
+                    cpl_array_set(params, 4, 1.0 + (IM13 - 1) * 0.5 + yfrac);
+
+                    code = cpl_fit_image_gaussian(image, use_errim, IM13/2,
+                                                  IM13/2, IM13, IM13, params,
+                                                  NULL, NULL, &rms1, NULL, NULL,
+                                                  &major1, &minor1, &angle1,
+                                                  NULL);
+
+                    cpl_test_eq_error(code, CPL_ERROR_NONE);
+                } else if (idoerrim) {
+                    cpl_test_eq_error(code, CPL_ERROR_NONE);
+                    cpl_test_nonnull(phys_cov);
+                    cpl_matrix_delete(phys_cov);
+                } else {
+                    cpl_test_eq_error(code, CPL_ERROR_NONE);
+                    cpl_test_null(phys_cov);
+
+                    code = cpl_fit_image_gaussian(image, NULL, IM13/2, IM13/2,
+                                                  IM13, IM13, params, NULL,
+                                                  NULL, &rms1, NULL, NULL,
+                                                  &major1, &minor1, &angle1,
+                                                  &phys_cov);
+                    cpl_test_eq_error(code, CPL_ERROR_DATA_NOT_FOUND);
+                }
+            }
+
+            cpl_msg_info(cpl_func, CPL_STRINGIFY(IM13) " X " CPL_STRINGIFY(IM13)
+                         " double-image: Major: %g - %g = %g. Minor: %g - %g "
+                         "= %g. Angle: %g - %g = %g. RMS=%g - %g = %g",
+                         major1, major0, major1 - major0,
+                         minor1, minor0, minor1 - minor0,
+                         angle1, angle0, angle1 - angle0,
+                         rms1,   rms0,   rms1 - rms0);
+
+            if (idobpm) {
+                /* With bpm, the change is less or equal. FIXME: Why ? */
+                cpl_test_leq(fabs(dmajor), fabs(major1 - major0));
+                cpl_test_leq(fabs(dminor), fabs(minor1 - minor0));
+                cpl_test_leq(fabs(dangle), fabs(angle1 - angle0));
+                cpl_test_leq(fabs(drms),   fabs(rms1   - rms0));
+            }
+
+            code = cpl_image_accept_all(image);
+            cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+            dmajor = major1 - major0;
+            dminor = minor1 - minor0;
+            dangle = angle1 - angle0;
+            drms   = rms1   - rms0;
+        }
+    }
+
+    cpl_array_delete(params);
+    cpl_image_delete(image);
+    cpl_image_delete(errim);
+}
+
 /*----------------------------------------------------------------------------*/
 /**
   @internal
@@ -1796,18 +1990,25 @@ static void cpl_fit_image_gaussian_test_local(void)
                             -0.556366};
 
     const size_t n = sizeof(values)/sizeof(values[0]);
-    size_t i;
 
-    const double tol = 1e-5;
+    const size_t image_bytes = IMSZ * IMSZ * sizeof(int);
+    int        * image_data  = (int*)cpl_malloc(image_bytes);
+    cpl_image  * image       = cpl_image_wrap_int(IMSZ, IMSZ, image_data);
 
-    for (i = 0; i < n; i++) {
+    cpl_test_nonnull(image);
+
+    cpl_test_assert(sizeof(major)/sizeof(major[0]) == n);
+    cpl_test_assert(sizeof(minor)/sizeof(minor[0]) == n);
+    cpl_test_assert(sizeof(angle)/sizeof(angle[0]) == n);
+
+    for (size_t i = 0; i < n; i++) {
+        const double tol = 1e-5;
         const cpl_size nx = IMSZ;
         const cpl_size ny = IMSZ;
-        cpl_image * image = cpl_image_wrap_int(nx, ny, (int*)values[i]);
         double mymajor, myminor, myangle, rms;
         cpl_error_code error;
 
-        cpl_test_nonnull(image);
+        cpl_test_eq_ptr(memcpy(image_data, values[i], image_bytes), image_data);
 
         error = cpl_array_fill_window_invalid(params, 1, 7);
         cpl_test_eq_error(error, CPL_ERROR_NONE);
@@ -1818,16 +2019,38 @@ static void cpl_fit_image_gaussian_test_local(void)
         cpl_test_eq_error(error, CPL_ERROR_NONE);
 
         cpl_msg_info(cpl_func, CPL_STRINGIFY(IMSZ) " X " CPL_STRINGIFY(IMSZ)
-                     " int-image %d/%d: Major: %g <=> %g. Minor: %g <=> %g. "
-                     "Angle: %g <=> %g. RMS=%g", (int)i + 1, (int)n, mymajor,
-                     major[i], myminor, minor[i], myangle, angle[i], rms);
+                     " int-image %d/%d: Major: %g - %g = %g. Minor: %g - %g "
+                     "= %g. Angle: %g - %g = %g. RMS=%g", (int)i + 1, (int)n,
+                     mymajor, major[i], mymajor - major[i],
+                     myminor, minor[i], myminor - minor[i],
+                     myangle, angle[i], myangle - angle[i], rms);
+
+        cpl_test_abs(mymajor, major[i], tol);
+        cpl_test_abs(myminor, minor[i], tol);
+        cpl_test_abs(myangle, angle[i], tol);
+
+        if (i == 0) continue; /* Converges to a non-global minimum... */
+
+        /* Re-try with the solution as 1st guess */
+        error = cpl_fit_image_gaussian(image, NULL, nx/2, ny/2, nx, ny,
+                                       params, NULL, NULL, &rms, NULL, NULL,
+                                       &mymajor, &myminor, &myangle, NULL);
+        cpl_test_eq_error(error, CPL_ERROR_NONE);
+
+        cpl_msg_info(cpl_func, CPL_STRINGIFY(IMSZ) " X " CPL_STRINGIFY(IMSZ)
+                     " int-image %d/%d: Major: %g - %g = %g. Minor: %g - %g "
+                     "= %g. Angle: %g - %g = %g. RMS=%g (retry)",
+                     (int)i + 1, (int)n,
+                     mymajor, major[i], mymajor - major[i],
+                     myminor, minor[i], myminor - minor[i],
+                     myangle, angle[i], myangle - angle[i], rms);
 
         cpl_test_abs(mymajor, major[i], tol);
         cpl_test_abs(myminor, minor[i], tol);
         cpl_test_abs(myangle, angle[i], tol);
 
-        cpl_test_eq_ptr(cpl_image_unwrap(image), values[i]);
     }
 
     cpl_array_delete(params);
+    cpl_image_delete(image);
 }
