@@ -188,6 +188,9 @@ int main(void)
                                    0.1, 1, &mdata, &mpattern, NULL, NULL);
 
     cpl_test_error(CPL_ERROR_NONE);
+    cpl_test_nonnull(matches);
+    cpl_test_nonnull(mpattern);
+    cpl_test_nonnull(mdata);
     cpl_test_eq(cpl_matrix_get_ncol(mpattern), 3);
     cpl_test_eq(cpl_array_get_size(matches), 3);
 
@@ -206,6 +209,9 @@ int main(void)
                                    0.1, 1, &mdata, &mpattern, NULL, NULL);
 
     cpl_test_error(CPL_ERROR_NONE);
+    cpl_test_nonnull(matches);
+    cpl_test_nonnull(mpattern);
+    cpl_test_nonnull(mdata);
     cpl_test_eq(cpl_matrix_get_ncol(mpattern), 3);
     cpl_test_eq(cpl_array_get_size(matches), 3);
 
@@ -233,6 +239,9 @@ int main(void)
                                    0.1, 1, &mdata, &mpattern, NULL, NULL);
 
     cpl_test_error(CPL_ERROR_NONE);
+    cpl_test_nonnull(matches);
+    cpl_test_nonnull(mpattern);
+    cpl_test_nonnull(mdata);
     cpl_test_eq(cpl_matrix_get_ncol(mpattern), 3);
     cpl_test_eq(cpl_array_get_size(matches), 3);
 
@@ -251,6 +260,9 @@ int main(void)
                                    0.1, 1, &mdata, &mpattern, NULL, NULL);
 
     cpl_test_error(CPL_ERROR_NONE);
+    cpl_test_nonnull(matches);
+    cpl_test_nonnull(mpattern);
+    cpl_test_nonnull(mdata);
     cpl_test_eq(cpl_matrix_get_ncol(mpattern), 3);
     cpl_test_eq(cpl_array_get_size(matches), 3);
 
@@ -311,6 +323,9 @@ int main(void)
                                    0.1, 1, &mdata, &mpattern, NULL, NULL);
 
     cpl_test_error(CPL_ERROR_NONE);
+    cpl_test_nonnull(matches);
+    cpl_test_nonnull(mpattern);
+    cpl_test_nonnull(mdata);
     cpl_test_eq(cpl_matrix_get_ncol(mpattern), 8);
     cpl_test_eq(cpl_array_get_size(matches), 8);
 
@@ -330,6 +345,9 @@ int main(void)
                                    0.1, 1, &mdata, &mpattern, NULL, NULL);
 
     cpl_test_error(CPL_ERROR_NONE);
+    cpl_test_nonnull(matches);
+    cpl_test_nonnull(mpattern);
+    cpl_test_nonnull(mdata);
     cpl_test_eq(cpl_matrix_get_ncol(mpattern), 7);
     cpl_test_eq(cpl_array_get_size(matches), 8);
     cpl_test_eq(cpl_array_count_invalid(matches), 1);
@@ -360,6 +378,9 @@ int main(void)
                                    0.1, 1, &mdata, &mpattern, NULL, NULL);
 
     cpl_test_error(CPL_ERROR_NONE);
+    cpl_test_nonnull(matches);
+    cpl_test_nonnull(mpattern);
+    cpl_test_nonnull(mdata);
     cpl_test_eq(cpl_matrix_get_ncol(mpattern), 8);
     cpl_test_eq(cpl_array_get_size(matches), 8);
 
@@ -411,6 +432,9 @@ int main(void)
                                    0.1, 0.1, &mdata, &mpattern, NULL, NULL);
 
     cpl_test_error(CPL_ERROR_NONE);
+    cpl_test_nonnull(matches);
+    cpl_test_nonnull(mpattern);
+    cpl_test_nonnull(mdata);
     cpl_test_eq(cpl_matrix_get_ncol(mpattern), 70);
     cpl_test_eq(cpl_array_get_size(matches), 70);
 
@@ -466,6 +490,9 @@ int main(void)
                                    0.1, 0.1, &mdata, &mpattern, NULL, NULL);
 
     cpl_test_error(CPL_ERROR_NONE);
+    cpl_test_nonnull(matches);
+    cpl_test_nonnull(mpattern);
+    cpl_test_nonnull(mdata);
     cpl_test_eq(cpl_matrix_get_ncol(mpattern), npattern);
     cpl_test_eq(cpl_array_get_size(matches), npattern);
 
@@ -526,6 +553,9 @@ int main(void)
                                    0.1, 0.1, &mdata, &mpattern, NULL, NULL);
 
     cpl_test_error(CPL_ERROR_NONE);
+    cpl_test_nonnull(matches);
+    cpl_test_nonnull(mpattern);
+    cpl_test_nonnull(mdata);
     cpl_test_eq(cpl_matrix_get_ncol(mpattern), ndata);
     cpl_test_eq(cpl_array_get_size(matches), npattern);
 
@@ -568,6 +598,9 @@ int main(void)
                                    0.1, 0.1, &mdata, &mpattern, NULL, NULL);
 
     cpl_test_error(CPL_ERROR_NONE);
+    cpl_test_nonnull(matches);
+    cpl_test_nonnull(mpattern);
+    cpl_test_nonnull(mdata);
     cpl_test_eq(cpl_matrix_get_ncol(mpattern), npattern);
     cpl_test_eq(cpl_array_get_size(matches), npattern);
 
@@ -583,6 +616,9 @@ int main(void)
                                    0.1, 0.1, &mdata, &mpattern, NULL, NULL);
 
     cpl_test_error(CPL_ERROR_NONE);
+    cpl_test_nonnull(matches);
+    cpl_test_nonnull(mpattern);
+    cpl_test_nonnull(mdata);
     cpl_test_eq(cpl_matrix_get_ncol(mpattern), ndata);
     cpl_test_eq(cpl_array_get_size(matches), ndata);
 
